log('common/variables loaded');

funits = (function (global) {
    var State = (function (funitType, jocket) {
        this.funitType = funitType;
        this.jocket = jocket;
        this.invalid = jocket['invalid'] === true;
        this.value = jocket.value;

        this._fixedValue = function (value) {
            return this.funit && this.funit.fraction !== undefined ? value.toFixed(this.funit.fraction) : value;
        };


        if (this.funit && this.funit.parser !== undefined) {
            this.divs = this.funit.parser(this);
        }
        if (!this.divs || this.divs.size === 0) {
            this.divs = new Map();

            if (this.value && typeof this.value === 'object') {
                try {
                    var keys = Object.keys(this.value);
                }
                catch (e) {
                    console.log(e);
                    return;
                }
                for (var i = 0; i < keys.length; i++) {
                    this.divs.set(this.funitType + '_' + keys[i], this.invalid ? '-' : this._fixedValue(this.value[keys[i]]));
                }
            } else {
                this.divs.set(this.funitType, this.invalid ? '-' : this._fixedValue(this.value));
            }
        }
    });

    State.prototype = {
        get funit() { return MercuryElectricMeter[this.funitType]; },
        get isAmperageRateDependent() { return this.funit && this.funit.amperageDependent},
        get isVoltageRateDependent() { return this.funit && this.funit.voltageDependent},
    };

    var Funit = (function() {});
    Funit.prototype = {};

    var Info = new Funit();
    Info.funitType = 'Info';

    var SerialDate = new Funit();
    SerialDate.funitType = 'SerialDate';

    var RateAmperage = new Funit();
    RateAmperage.funitType = 'RateAmperage';

    var RateVoltage = new Funit();
    RateVoltage.funitType = 'RateVoltage';

    var PowerTotal = new Funit();
    PowerTotal.funitType = 'PowerTotal';
    PowerTotal.fraction = 2;
    PowerTotal.amperageDependent = true;
    PowerTotal.voltageDependent = true;

    var PowerActive = new Funit();
    PowerActive.funitType = 'PowerActive';
    PowerActive.fraction = 2;
    PowerActive.amperageDependent = true;
    PowerActive.voltageDependent = true;

    var PowerReactive = new Funit();
    PowerReactive.funitType = 'PowerReactive';
    PowerReactive.fraction = 2;
    PowerReactive.amperageDependent = true;
    PowerReactive.voltageDependent = true;

    var Voltage = new Funit();
    Voltage.funitType = 'Voltage';
    Voltage.fraction = 2;
    Voltage.voltageDependent = true;

    var Amperage = new Funit();
    Amperage.funitType = 'Amperage';
    Amperage.fraction = 2;
    Amperage.amperageDependent = true;

    var PowerRate = new Funit();
    PowerRate.funitType = 'PowerRate';
    PowerRate.fraction = 3;

    var Frequency = new Funit();
    Frequency.funitType = 'Frequency';
    Frequency.fraction = 2;

    var Angle = new Funit();
    Angle.funitType = 'Angle';
    Angle.fraction = 2;

    var CurrentDateTime = new Funit();
    CurrentDateTime.funitType = 'CurrentDateTime';
    CurrentDateTime.parser = function(state) {
        if (!state.invalid) {
            state.time = Date.parse(state.jocket.value.time);
            state.weekday = state.jocket.value.weekday;
            state.dst = state.jocket.value.dst;
        }
    };

    var StoredEnergy = new Funit();
    StoredEnergy.funitType = 'StoredEnergy';
    StoredEnergy.fraction = 3;
    StoredEnergy.amperageDependent = true;
    StoredEnergy.voltageDependent = true;
    StoredEnergy.parser = function(state) {
        var divs = new Map();
        if (state.invalid) {
            const periods = ['all', 'year', 'month', 'day'];
            for (var i = 0; i < periods.length; i++) {
                for (var j = 0; j < 5; j++) {
                    divs.set('s_energy_' + periods[i] + '_as_' + j, '-');
                }
            }
        } else {
            var period = state.jocket.params.period;
            var tariff = state.jocket.params.tariff;
            var periodStr;
            switch (Number(period)) {
                case 0:
                    periodStr = 'all';
                    break;
                case 1:
                case 2:
                    periodStr = 'year';
                    break;
                case 3:
                    periodStr = 'month';
                    break;
                case 4:
                case 5:
                    periodStr = 'day';
                    break;
                default:
                    periodStr = period;
            }
            if (state.jocket.value.hasOwnProperty('active')) {
                divs.set('s_energy_' + periodStr + '_as_' + tariff, state.jocket.value['active'].toFixed(StoredEnergy.fraction));
            }
            if (state.jocket.value.hasOwnProperty('active_reverse')) {
                divs.set('s_energy_' + periodStr + '_ar_' + tariff, state.jocket.value['active_reverse'].toFixed(StoredEnergy.fraction));
            }
            if (state.jocket.value.hasOwnProperty('reactive')) {
                divs.set('s_energy_' + periodStr + '_rs_' + tariff, state.jocket.value['reactive'].toFixed(StoredEnergy.fraction));
            }
            if (state.jocket.value.hasOwnProperty('reactive_reverse')) {
                divs.set('s_energy_' + periodStr + '_rr_' + tariff, state.jocket.value['reactive_reverse'].toFixed(StoredEnergy.fraction));
            }
        }
        return divs;
    };

    var PulseMode = new Funit();
    PulseMode.funitType = 'PulseMode';

    var Status = new Funit();
    Status.funitType = 'Status';

    var LoadOn = new Funit();
    LoadOn.funitType = 'LoadOn';

    var CurrentLevelRaw = new Funit();
    CurrentLevelRaw.funitType = 'CurrentLevelRaw';

    var BrightnessLevel = new Funit();
    BrightnessLevel.funitType = 'BrightnessLevel';

    var FadeTime = new Funit();
    FadeTime.funitType = 'FadeTime';

    var FadeRate = new Funit();
    FadeRate.funitType = 'FadeRate';

    var SceneLevelsRaw = new Funit();
    SceneLevelsRaw.funitType = 'SceneLevelsRaw';

    var Groups = new Funit();
    Groups.funitType = 'Groups';

    var MercuryElectricMeter = {
        RateVoltage: RateVoltage,
        StoredEnergy: StoredEnergy,
        CurrentDateTime: CurrentDateTime,
        Frequency: Frequency,
        PowerRate: PowerRate,
        PowerActive: PowerActive,
        PowerReactive: PowerReactive,
        PowerTotal: PowerTotal,
        Angle: Angle,
        Amperage: Amperage,
        Voltage: Voltage,
    };

    return {
        State: State,
        Info: Info,
        SerialDate: SerialDate,
        RateVoltage: RateVoltage,
        RateAmperage: RateAmperage,
        PowerTotal: PowerTotal,
        PowerActive: PowerActive,
        PowerReactive: PowerReactive,
        Voltage: Voltage,
        Amperage: Amperage,
        PowerRate: PowerRate,
        Frequency: Frequency,
        Angle: Angle,
        CurrentDateTime: CurrentDateTime,
        StoredEnergy: StoredEnergy,
        PulseMode: PulseMode,
        Status: Status,
        LoadOn: LoadOn,
        CurrentLevelRaw: CurrentLevelRaw,
        BrightnessLevel: BrightnessLevel,
        FadeTime: FadeTime,
        FadeRate: FadeRate,
        SceneLevelsRaw: SceneLevelsRaw,
        Groups: Groups,
    };
})(window);