var client;
var command;
var entityId;

function onReady(config) {
    command = config.project.command;
    entityId = config.project.entity_id;
    client = init_client(config.broker, onConnect, onUpdate);

    function onConnect() {
        console.log('connected');
        client.subscribe(config.project.state);
    }
}

function sendJocket(funitType, data, isSet) {
    var jocket = {
        address: {id: entityId, class: 0},
        action: isSet ? 'set' : 'get',
        key: funitType,
        data: data?data:{}
    };

    var topic = command.replace('{}', funitType);

    client.send(topic, JSON.stringify(jocket));
    console.log(`command: [${topic}]: ${JSON.stringify(jocket.data)}`);
}

function onUpdate(topic, jocket) {
    if (jocket.hasOwnProperty('error')) {
        console.log("Error: " + JSON.stringify(jocket.error));
        return;
    }
    topic = topic.split('/');
    var state = null;

    if (topic[0] === 'Jocket')
        if (jocket.hasOwnProperty("error")) {
            console.log(JSON.stringify(jocket.error));
        } else if (topic[1] === 'State') {
            state = new funits.State(topic.slice(-1)[0], jocket);
        } else if (topic[1] === 'Reply') {
            state = new funits.State(jocket.key, jocket);
        }

    if (state) {
        if (state.funitType === funits.SceneLevelsRaw.funitType) {
            for (var i = 0; i <= 15; i++) {
                try {
                    const include = state.value.length > i && state.value[i].value !== 255;
                    $('#scene_' + state.value[i].index + '_include')[0].checked = include;
                    const input = $('#scene_' + state.value[i].index + '_level')[0];
                    input.hidden = !include;
                    if (include) {
                        const value = state.value[i]['value'];
                        input.value = value;
                        input.oninput(value);
                    }
                } catch (e) {

                }
            }
        } else if (state.funitType === funits.Groups.funitType) {
            for (var i = 0; i <= 15; i++) {
                try {
                    const include = state.value.length > i && state.value[i].value;
                    const gr = $('#group_' + state.value[i].index)[0];
                    gr.checked = include;
                    if (include) {
                        if (gr.labels[0].className.indexOf(' member') === -1) {
                            gr.labels[0].className += ' member';
                        }
                    } else {
                        gr.labels[0].className = gr.labels[0].className.replace(' member', '');
                    }
                } catch (e) {

                }
            }
        } else {
            for (var divName of state.divs) {
                var value = divName[1];

                if (typeof document != 'undefined') {
                    var div = document.getElementById(divName[0]);
                    if (div === null) {
                        // console.log("DIV is null! " + divName[0] + ": " + value);
                    } else if (div.tagName === 'INPUT') {
                        if (div.type === 'checkbox') {
                            div.checked = value;
                        } else {
                            div.value = value;
                        }
                    } else if (div.tagName === 'SELECT') {
                        div.value = value;
                    } else {
                        // console.log(divName[0] + ": " + value);
                        div.textContent = value;
                    }
                }
            }
        }
    }
}

function setValue(input, action) {
    try {
        sendJocket(input.id, {value: !isNaN(input.value) ? Number(input.value) : input.value}, action!=='get');
    }
    catch (e) {
        console.error(e);
    }
    return false;
}

function setSceneLevel(sceneId, level) {
    if (typeof level === "boolean") {
        var input = $('#scene_' + sceneId + '_level')[0];
        input.labels[0].textContent = '';
        input.hidden = !level;

        if (level) {
            sendJocket(funits.SceneLevelsRaw.funitType, {value: [{index: sceneId, value: Number(input.value)}]}, true);
        }
        else {
            sendJocket(funits.SceneLevelsRaw.funitType, {value: [{index: sceneId, value: 255}]}, true);
        }
    }
    else {
        sendJocket(funits.SceneLevelsRaw.funitType, {value: [{index: sceneId, value: Number(level)}]}, true);
    }
}

function setGroup(groupId, include) {
    if (include.labels[0].className.indexOf(' member') === -1) {
        include.labels[0].className += ' member';
    } else {
        include.labels[0].className = include.labels[0].className.replace(' member', '');
    }
    sendJocket(funits.Groups.funitType, {value: [{index: groupId, value: include.checked}]}, true);
}

$(document).on('click', '.panel-heading', function () {
    $(this).parents('.panel').find('.panel-body').slideToggle();
    $(this).find('i').toggleClass('glyphicon-chevron-down').toggleClass('glyphicon-chevron-up');
});