const R = new RegExp(STATES
    .replace('/Location/+/', '/Location/(\\d+)/')
    .replace('/+', '/(\\w+)')
);

var cycles = {};
var locations = {};

const client = init_client(
    broker,
    function onConnect() {
        console.log('MQTT Connected');
        client.subscribe(LOCATIONS, {});
        client.subscribe(CYCLES + '+', {});
        client.subscribe(STATES, {});
    },
    function onMessage(topic, message) {
        console.log('Recv: ', topic, message);
        if (topic === LOCATIONS) {
            updateLocations(message['locations']);
        } else if (topic.startsWith(CYCLES)) {
            const id = topic.substr(topic.lastIndexOf('/') + 1);
            updateCycles(id, message);
        } else {
            const match = topic.match(R);
            if (match && match.length > 0) {
                updateIncubator(match[1], match[2], message);
            }
        }
    }
);

function updateIncubator(id, key, value) {
    const $table = $('.locations table');

    var $rows = $table.find('#' + id);

    if ($rows.length === 0) {
        if (key !== "Cycle" || !value) return;

        const row = $table[0].insertRow();
        row.innerHTML = $('#template-incubator')[0].innerHTML;
        row.id = id;
        row.classList.add('location');
        $(row).find('.loc-name').text(locations[id]);
        $(row).find('select[name=day]').on('change', e => {
            const date = new Date(new Date() - e.currentTarget.value * 24*60*60*1000);
            request($(row)[0], 'StartTime', date.toISOString().split('T')[0])
        });
        // Изменение отображения кнопок play/pause
        $(row).find('input[name=active]')
            .on('change', e => request($(e.target).parents('.location')[0], 'Active', e.target.checked))
            .on('incoming', function (e) {
                if (e.target.checked) {
                    $(e.target.parentElement).find('.glyphicon-play').removeClass('unselected');
                    $(e.target.parentElement).find('.glyphicon-pause').addClass('unselected');
                } else {
                    $(e.target.parentElement).find('.glyphicon-play').addClass('unselected');
                    $(e.target.parentElement).find('.glyphicon-pause').removeClass('unselected');

                }

            })
            .parents('label').on('click', function (e) {
            // обработка клика на play/pause
            if (e.target.classList.contains('glyphicon') && !e.target.classList.contains('unselected'))
                e.preventDefault();
        });

        $(row).find('input[name=recursive]')
            .on('change', e => request($(e.target).parents('.location')[0], 'Recursive', e.target.checked))
            .on('incoming', function (e) {
                const value = e.target.checked;
                if (value) {
                    e.target.parentElement.classList.remove('unselected');
                } else {
                    e.target.parentElement.classList.add('unselected');
                }

                $(row).find(' select[name=day]').each((_, day) => {
                    if (day.hasAttribute('cd') && $rows.attr('cycle')) {
                        const cd = day.getAttribute('cd');
                        day.value = value ? (cd % cycles[$rows.attr('cycle')].period) : cd;
                    }
                });
            });
        $rows.push(row);
    }

    if (key === 'Cycle') {
        if (!value) {
            $rows.remove();
            $(`#new-incubator select[name=location] option[value=${$rows[0].id}]`).show();
        } else {

            $(`#new-incubator select[name=location] option[value=${id}]`).hide();

            if (!cycles.hasOwnProperty(value.value)) return;

            $rows.find('.cycle-name').text(cycles[value.value].name);
            $rows.attr('cycle', value.value);
            $rows.find(' select[name=day]').each((_, selector) => {
                $(selector).find('option').remove();
                for (var i = -1; i < cycles[value.value].period; i++) {
                    const opt = document.createElement('option');
                    if (i === -1) {
                        opt.selected = true;
                        opt.hidden = true;
                    } else {
                        opt.value = i;
                        opt.textContent = 'День ' + (i + startIndex);
                    }
                    selector.append(opt);
                }
            });
        }

    } else if (key === 'StartTime') {
        const date = Date.parse(value.value);
        const today = Date.parse((new Date()).toISOString().split('T')[0]);
        const cd = (today - date) / 1000/60/60/24;
        $rows.find(' select[name=day]').each((_, selector) => {
            selector.setAttribute('cd', cd);
            const recursive = $rows.find('input[name=recursive]')[0].checked;
            selector.value = recursive ? (cd % cycles[$rows.attr('cycle')].period) : cd;
        });
    } else if (key === 'Active') {
        $rows.find(' input[name=active]').each((_, cb) => {
            cb.checked = value.value;
            cb.dispatchEvent(new Event('incoming', {target: cb}))
        });
    } else if (key === 'Recursive') {
        $rows.find(' input[name=recursive]').each((_, cb) => {
            cb.checked = value.value;
            cb.dispatchEvent(new Event('incoming', {target: cb}))
        });
    }
}

function updateLocations(data) {
    const selector = $('#new-incubator select[name=location]');

    for (var location of data) {
        const optId = 'loc-' + location.id;
        var $option = selector.find(`#${optId}`);
        if ($option.length === 0) {
            const opt = document.createElement('option');
            opt.id = optId;
            opt.value = location.id;
            selector.append(opt);
            $option.push(opt);
        }

        $option.text(location.name.replace('*', ''));
        locations[location.id] = $option.text();
    }
}

function updateCycles(id, cycle) {
    const selector = $('#new-incubator select[name=cycle]');

    const optId = 'cycle-' + id;
    var $option = selector.find(`#${optId}`);
    if ($option.length === 0) {
        const opt = document.createElement('option');
        opt.id = optId;
        opt.value = id;
        selector.append(opt);
        $option.push(opt);
    }

    $option.text(cycle.name);
    cycles[id] = cycle;
}

function createIncubator(form) {
    if (form.checkValidity()) {
        const topic = STATES
            .replace('State', 'Set')
            .replace('+', form.location.value)
            .replace('+', 'Cycle');

        client.send(topic, {value: parseInt(form.cycle.value)}, 0, false);
        client.send(topic.replace('Cycle', 'StartTime'), {value: (new Date()).toISOString().split('T')[0]}, 0, false);
        setTimeout((form) => {
            form.location.value = '';
            form.cycle.value = '';
        }, 100, form);
    }
}

function removeIncubator(btn) {
    showModal('Удалить помещение?', ()=> {
        $(btn).parents('.location').each((i, row) => request(row, 'Cycle'));
    });
}

function request(row, funit, value) {
    var data;

    if (value === undefined){
        data = '';
    } else {
        data = JSON.stringify({
            value: value
        });
    }
    const topic = STATES
        .replace('State', 'Set')
        .replace('+', row.id)
        .replace('+', funit);
    try {
        console.log(topic, data);
        client.send(topic, data, 0, false);
    } catch (e) {
        console.error(e);
    }
}