log('scheduler started');

const AUTO = gettext('scheduler_auto_mark');
const MONTH = 'month';
const SET_DAY = 'set_day';
const SET_MULTI_DAY = 'set_multi_day';
const SET_ENABLED = 'set_enabled';
const STATUS = 'status';
const GET_TRIGGERS = 'get_triggers';
const GET_TRIGGER_DIV = 'get_trigger_div';
const GET_ACTION_DIV = 'get_action_div';
const SET_TIME = 'set_time';
const SET_ACTION = 'set_action';
const REMOVE_TRIGGER = 'remove_trigger';
const REMOVE_ACTION = 'remove_action';

var locationId = null;
var entities = [];
var trigger_content = null;
var action_content = null;

function ready() {
    window.onresize = onResize;
    onResize();

    request(GET_TRIGGER_DIV, function (content) {
        trigger_content = content;
        request(GET_ACTION_DIV, function (content) {
            action_content = content;
            nextMonth();
            onSelect();
        }, null, null);
    }, null, null);
}

function nextMonth(m, y) {
    var url = MONTH;
    if (m&&y) {
        url = [url, m, y].join('/');
    }

    request(url,
        function(content) {
            var div = document.createElement('div');
            div.innerHTML = content;
            const parent = document.getElementById('calendar');
            if (parent.getElementsByClassName('r')[0]) {
                parent.getElementsByClassName('r')[0].remove();
            }
            parent.appendChild(div);
        }, null, null);
}

function changeDay(target, y, m, d) {
    request(SET_DAY,
        function (isWeekend) {
            if (isWeekend) {
                target.classList.add('weekend');
                target.classList.remove('work');
            }
            else {
                target.classList.add('work');
                target.classList.remove('weekend');
            }
        },
        function () {
            shake(target);
        },
        {year: y, month: m, day: d}
    );
}

function changeMultiDay(target, y, m, d) {
    showModal(gettext('scheduler_set_holiday_modal'), function() {
        request(SET_MULTI_DAY, function (isWeekend) {
                $(target).parents(' .calendar-table').find(' .week-day-' + d).each(function(i, day) {
                    if (isWeekend) {
                        day.classList.add('weekend');
                        day.classList.remove('work');
                    } else {
                        day.classList.add('work');
                        day.classList.remove('weekend');
                    }
                });
            },
            function () {
                shake(target);
            },
            {year: y, month: m, day: d}
        );
    });
}

function onSelect() {
    const selector = document.getElementById('selector');
    if (selector.selectedIndex !== -1) {
        locationId = selector.value;

        request(STATUS,
            function (response) {
                entities = response.entities;
                setToggle(response.enabled);
                drawActions();
                $('#control-panel').hide().find('tr').remove();
            },
            null,
            {id: locationId}
        );
    } else {
        $('#slidertable').hide();
    }
}

function onTabClick(e) {
    const target = e.target.tagName === 'BUTTON' ? e.target.parentElement : e.target;
    const collection = document.getElementsByClassName('panel-selected-td');
    for (var i = 0; i < collection.length; i++) {
        if (collection[i] === target) {
            return;
        }
        collection[i].className = 'panel-not-selected-td';
    }

    target.className = 'panel-selected-td';

    drawActions();
}

function drawActions() {
    const dayType = document.getElementsByClassName('panel-selected-td')[0].id;
    const table = document.getElementsByClassName('panel-table')[0];
    var rows = table.children[0];
    while (rows.childElementCount > 2) {
        rows.children[1].remove();
    }

    request(GET_TRIGGERS,
        function (triggers) {
            triggers = triggers.sort(function (a, b) {
                return a.time.localeCompare(b.time);
            });
            for (var i=0; i < triggers.length; i++) {
                var trigger = createTrigger(
                    table.insertRow(table.rows.length - 1),
                    triggers[i]
                );

                for (var j=0; j < triggers[i].actions.length; j++) {
                    var a_table = trigger.getElementsByClassName('b')[0];
                    createAction(
                        a_table.insertRow(a_table.rows.length - 1),
                        triggers[i].actions[j]
                    );
                }
            }
        },
        null,
        {id: locationId, trigger: dayType}
    );
}

function onToggleHelper(e) {
    const slider = document.getElementById("autocheckbox");
    slider.checked = !slider.checked;
    onToggle({target: slider});
}

function setToggle(enabled) {
    document.getElementsByClassName('panel-table')[0].hidden = !enabled;
    document.getElementById('autocheckbox').checked = enabled;
    const selector = document.getElementById('selector');
    const option = selector.options[selector.selectedIndex];

    if (enabled) {
        if (option.textContent.indexOf(AUTO) === -1) {
            option.textContent += AUTO;
        }
    } else {
        if (option.textContent.indexOf(AUTO) !== -1) {
            option.textContent = option.textContent.substring(0, option.textContent.indexOf(AUTO));
        }
    }
}

function onToggle(e) {
    const enabled = e.target.checked;
    request(
        SET_ENABLED,
        function () {
            setToggle(enabled);
        },
        function (e) {
            setToggle(!enabled);
        },
        {id: locationId, enabled: enabled}
    );
}

function createTrigger(row, data) {
    var div = document.createElement('td');
    div.className = "panel-td";
    div.setAttribute('colspan', '2');
    div.innerHTML = trigger_content;
    if (data !== undefined) {
        div.getElementsByClassName('shcedule-time')[0].value = data.time;
        row.setAttribute('trigger-id', data.id);
        row.setAttribute('trigger-time', data.time);
    }

    row.appendChild(div);

    fillTargets($(div).find('.shcedule-select')[0]);

    return div;
}

function addAction(select) {
    const table = $(select).closest('table')[0];
    const timeDiv = $(table).find('.shcedule-time')[0];
    if (select.selectedIndex === 0) {
        shake(select)
    } else if (!timeDiv.value) {
        shake(timeDiv);
    } else if (!$(timeDiv).closest(' table').closest(' tr')[0].getAttribute('trigger-id')) {
        shake($(timeDiv.form).find( 'input[name=commit]')[0]);
    } else {
        createAction(table.insertRow(table.rows.length - 1), {
            eId: parseInt(select.options[select.selectedIndex].getAttribute('c-entity'))
        });

        if (++select.selectedIndex >= select.options.length) {
            select.selectedIndex = 0;
        }
    }
}

function removeAction(btn, row) {
    if (row === undefined) {
        row = $(btn).closest('.action')[0]
    }

    if (row.hasAttribute('action-id')) {
        request(REMOVE_ACTION,
            function () {
                destroy(row);
                onRowChecked(
                    $(row).find('.select-row input:checkbox').prop('checked', false)[0]
                );
            },
            function () {
                shake(row);
            },
            {
                location: locationId,
                trigger: $(row).closest('.action').attr('trigger-id'),
                action: $(row).attr('action-id')
            });
    }
    else {
        destroy(row);
        onRowChecked(
            $(row).find('.select-row input:checkbox').prop('checked', false)[0]
        );
    }
}

function removeControl(e) {
    $('#commit-panel')
        .show()
        .css('top', e.originalEvent.clientY - 60)
        .css('left', e.originalEvent.clientX - 100)
        .find('#commit-yes').one('click',() => {
        const actionTarget = $(e.currentTarget).closest('tr').find('.shcedule-select.target')[0].value;
        $('#panel-table')
            .find('.select-row input:checkbox:checked')
            .closest('.action[action-target=' + actionTarget + ']')
            .each(removeAction);
    });
}

function onRowChecked(cb) {
    const panel = $('#control-panel');
    const panelTable = $('#panel-table');
    const actionTarget = $(cb).closest('.action').attr('action-target');
    const count = panelTable.find('.action .select-row input:checkbox:checked');
    const currentCount = count.closest('.action[action-target=' + actionTarget + ']');

    if (cb) {
        var tr = panel.find('.shcedule-select.target option[value=' + actionTarget + ']').closest('tr');
        if (!cb.checked) {
            if (currentCount.length === 0) {
                tr.remove();
            }
        } else {
            if (tr.length === 0) {
                tr = $(panel.find('table')[0].insertRow());

                $(cb).closest('.select-row').clone().appendTo(tr[0].insertCell())
                    .find('input:checkbox').prop('checked', false)
                    .removeAttr('onchange')
                    .on('change', function (e) {
                        $('#panel-table')
                            .find('.action[action-target=' + actionTarget + '] .select-row input:checkbox')
                            .prop('checked', e.currentTarget.checked).each((i, _cb) => onRowChecked(_cb));
                    });

                $(cb).closest('.action')
                    .find('.shcedule-select.target, .shcedule-select.command, .shcedule-select.value, .shcedule-input')
                    .each((i, node) => {
                        const s = node.cloneNode(true);
                        s.classList.add('control');
                        if (node.tagName === 'SELECT') {
                            tr[0].insertCell().appendChild(s);
                            s.selectedIndex = node.selectedIndex;
                        } else {
                            tr[0].cells[tr[0].cells.length - 1].appendChild(s);
                            s.value = node.value;
                        }

                        if (s.classList.contains('target')) {
                            s.disabled = true;
                            $(s).find(' option').each((i, opt) => {
                                if ($(opt).is(':selected')) {
                                    if (opt.value > 0) {
                                        opt.textContent = $(opt).closest(' optgroup').attr('label');
                                    }
                                } else {
                                    $(opt).remove();
                                }
                            });
                        }
                    });

                $(document.createElement('button'))
                    .attr('name',  'commit')
                    .text(gettext('scheduler_control_commit'))
                    .appendTo(tr[0].insertCell());

                $(document.createElement('button'))
                    .attr('name', 'remove')
                    .text(gettext('scheduler_control_remove').replace('{0}', currentCount.length))
                    .on('click', removeControl)
                    .appendTo(tr[0].insertCell());
            }
        }
        tr.find('button[name=remove]').text(
            gettext('scheduler_control_remove').replace('{0}', currentCount.length)
        );

        $(cb).closest('#time').each((i, table) => {
            const tableCount = $(table).find('.action .select-row input:checkbox');
            const tableCountChecked = $(table).find('.action .select-row input:checkbox:checked');

            $(table).find('.first-row .select-row input:checkbox')
                .prop('checked', tableCount.length > 0 && tableCount.length === tableCountChecked.length);
        });

        const targetCount = panelTable.find('.action[action-target=' + actionTarget + '] .select-row input:checkbox');
        tr.find('.select-row input:checkbox').prop('checked',
            currentCount.length > 0 && targetCount.length === currentCount.length);
    }

    if (panel.find('tr').length > 0) {
        panel.show();
    } else {
        panel.hide();
    }

    panel.find('button[name=remove-all]').text(
        gettext('scheduler_control_remove').replace('{0}', count.length)
    );
}

function selectTrigger(cb) {
    $(cb)
        .closest('#time')
        .find('.action .select-row input:checkbox')
        .prop('checked', cb.checked)
        .each((i, _cb) => onRowChecked(_cb));
}

function clearSelection() {
    $('#panel-table').find('.action .select-row input:checkbox:checked').click();
}

function removeAllSelections(e) {
    $('#commit-panel')
        .show()
        .css('top', e.clientY - 60)
        .css('left', e.clientX - 100)
        .find('#commit-yes').one('click',() => {
        $('#control-panel').find('button[name=remove]').each((i, btn) => {
            const actionTarget = $(btn).closest('tr').find('.shcedule-select.target')[0].value;
            $('#panel-table')
                .find('.select-row input:checkbox:checked')
                .closest('.action[action-target=' + actionTarget + ']')
                .each(removeAction);
        });
    });
}

function addTrigger(e) {
    const table = $(e).closest('td').closest('table')[0];
    createTrigger(
        table.insertRow(table.rows.length - 1)
    );
}

function hideTrigger(btn) {
    hide($(btn).closest('#time').find('.action, .action-creator'));
    $(btn).parent().find('.show-trigger').show();
    $(btn).parent().find('.hide-trigger').hide();
}

function showTrigger(btn) {
    show($(btn).closest('#time').find('.action, .action-creator'));
    $(btn).parent().find('.show-trigger').hide();
    $(btn).parent().find('.hide-trigger').show();
}

function onRemoveTrigger(e) {
    $('#commit-panel')
        .show()
        .css('top', e.clientY - 60)
        .css('left', e.clientX - 100)
        .find('#commit-yes').one('click',() => removeTrigger(e.target));
}

function removeTrigger(e) {
    const table = $(e).closest('table')[0];
    const row = $(table).closest('tr')[0];
    if (row.hasAttribute('trigger-id')) {
        request(REMOVE_TRIGGER,
            function () {
                destroy(table);
                $(e).closest('#time').find('.action').each((i, r) => {
                    onRowChecked(
                        $(r).find('.select-row input:checkbox').prop('checked', false)[0]
                    );
                });
            },
            function () {
                shake(e);
            },
            {
                location: locationId,
                trigger: row.getAttribute('trigger-id')
            }
        );
    } else {
        destroy(table);
        $(e).closest('#time').find('.action').each((i, r) => {
            onRowChecked(
                $(r).find('.select-row input:checkbox').prop('checked', false)[0]
            );
        });
    }
}

function createAction(row, data) {
    row.innerHTML = action_content;
    row.classList.add('action', 'animated-destroy');
    var targetSelector = $(row).find('.shcedule-select.target')[0];

    fillTargets(targetSelector, data);

    if (data !== undefined && data.hasOwnProperty('id')) {
        row.setAttribute('action-id', data.id);
    }
    else {
        shake($(row).find('.shcedule-select.command')[0]);
        $(row).closest('#time')
            .find(' .first-row .select-row input:checkbox')
            .prop('checked', false);

        $('#control-panel')
            .find('option[value=' + targetSelector.value + ']')
            .closest('tr')
            .find('.select-row input:checkbox')
            .prop('checked', false);
    }

    onSelectTarget(targetSelector, data);
}

function fillTargets(targetSelector, data) {
    for (var i=0; i < entities.length; i++) {
        var opt = document.createElement('option');
        opt.textContent = entities[i].name;
        opt.value = entities[i].entityTypeIndex;
        opt.setAttribute('c-entity', entities[i].entityId);

        const groupName = groups[entities[i].entityTypeIndex];
        if (groupName) {
            var group = $(targetSelector).find('optgroup.group-' + entities[i].entityTypeIndex)[0];
            if (!group) {
                group = document.createElement('optgroup');
                group.label = groupName;
                group.classList.add('group-' + entities[i].entityTypeIndex);
                targetSelector.appendChild(group);
            }
            group.appendChild(opt);
        } else {
            targetSelector.appendChild(opt);
        }

        if (data && data.eId === entities[i].entityId) {
            opt.selected = true;
        }
    }
}

function onSelectTarget(targetSelector, data) {
    const row = $(targetSelector).closest('tr')[0];
    const tIndex = targetSelector.value;
    const commandSelector = $(row).find('.command')[0];
    var options = commandSelector.getElementsByTagName('option');
    while (options.length > 1) {
        options[1].remove();
    }

    if (tIndex > -1) {
        const targets = Object.keys(commands);
        $(targetSelector).find('option[value=' + -1 + ']').prop('hidden', true);
        $(targetSelector).find('option[value=0]').prop('disabled', true);
        $(targetSelector).find('optgroup').prop('disabled', true);
        $(targetSelector).find('optgroup.group-' + tIndex).prop('disabled', false);
        row.setAttribute('entity-id', targetSelector.options[targetSelector.selectedIndex].getAttribute('c-entity'));
        for (var i = 0; i < commands[targets[tIndex]].length; i++) {
            var opt = document.createElement('option');
            opt.textContent = commands[targets[tIndex]][i].name;
            commandSelector.appendChild(opt);
        }

        row.setAttribute('action-target', tIndex);
    }

    if (data !== undefined) {
        commandSelector.selectedIndex = data.action + 1;
        row.setAttribute('action-command', commandSelector.selectedIndex);
        onSelectCommand(commandSelector, data);
    }
    else {
        commandSelector.selectedIndex = 0;
        shake(commandSelector);
        $(row).find('.shcedule-input')[0].hidden = true;
        $(row).find('.value')[0].hidden = true;
    }
}

function onSelectCommand(commandSelector, data) {
    const row = $(commandSelector).closest('tr')[0];
    const cIndex = commandSelector.selectedIndex-1;
    const targetSelector = $(row).find('.target')[0];
    const valueSelector = $(row).find('.value')[0];
    const valueInput = $(row).find('.shcedule-input')[0];
    const command = commands[targetSelector.value][cIndex];

    if (command === undefined) return;

    row.removeAttribute('action-value');
    valueInput.value = '';

    if (command.values === undefined) {
        valueSelector.hidden = true;
        valueInput.hidden = false;
        if (command.hasOwnProperty('validator')) {
            valueInput.min = command.validator.min;
            valueInput.max = command.validator.max;
        }
        onValueChanged(valueInput, data);
    }
    else if (command.values.length > 1) {
        valueSelector.hidden = false;
        valueInput.hidden = true;

        var options = valueSelector.getElementsByTagName('option');
        while (options.length > 1) {
            options[1].remove();
        }

        for (var i=0; i < command.values.length; i++) {
            var opt = document.createElement('option');
            opt.value = command.values[i][0];
            opt.textContent = command.values[i][1];
            valueSelector.appendChild(opt);
        }

        if (data !== undefined) {
            valueSelector.selectedIndex = data.value + 1;
            onSelectValue(valueSelector, data);
        }
        else {
            valueSelector.selectedIndex = 0;
            shake(valueSelector);
        }
    }
    else {
        valueSelector.hidden = true;
        valueInput.hidden = true;

        if (data === undefined) {
            if (commandSelector.classList.contains('control')) {
                $(valueInput).closest('tr').find('button[name=commit]')[0].onclick = function () {
                    $('#panel-table')
                        .find('.select-row input:checkbox:checked')
                        .closest('.action[action-target=' + targetSelector.value + ']')
                        .each((i, r) => {
                            onSelectCommand($(r).find('.command').val(commandSelector.value)[0]);
                        });
                };
            } else {
                request(SET_ACTION,
                    function (response) {
                        row.setAttribute('action-id', response);
                        row.setAttribute('action-command', cIndex);
                    },
                    function (error, data) {
                        commandSelector.selectedIndex = row.getAttribute('action-command');
                        shake(commandSelector);
                    },
                    {
                        eId: row.getAttribute('entity-id'),
                        target: targetSelector.value,
                        trigger: $(row).closest('table').closest('tr')[0].getAttribute('trigger-id'),
                        action: row.getAttribute('action-id'),
                        index: cIndex
                    });
            }
        }
    }
}

function onValueChanged(valueInput, data) {
    const row = $(valueInput).closest('tr')[0];
    const targetSelector = $(row).find('.target')[0];
    const commandSelector = $(row).find('.command')[0];

    if (data !== undefined) {
        valueInput.value = data.value;
    }
    else if (valueInput.value !== row.getAttribute('action-value'))  {
        if (valueInput.value) {
            if (commandSelector.classList.contains('control')) {
                $(valueInput).closest('tr').find('button[name=commit]')[0].onclick = function () {
                    $('#panel-table')
                        .find('.select-row input:checkbox:checked')
                        .closest('.action[action-target=' + targetSelector.value + ']')
                        .each((i, r) => {
                            onSelectCommand($(r).find('.command').val(commandSelector.value)[0]);
                            onValueChanged($(r).find('input').val(valueInput.value)[0]);
                        });
                };
            } else {
                request(SET_ACTION,
                    function (response) {
                        row.setAttribute('action-id', response);
                        row.setAttribute('action-value', valueInput.value);
                    },
                    function (error, data) {
                        valueInput.value = row.getAttribute('action-value');
                        shake(valueInput);
                    },
                    {
                        eId: row.getAttribute('entity-id'),
                        target: targetSelector.value,
                        trigger: $(row).closest('table').closest('tr')[0].getAttribute('trigger-id'),
                        action: row.getAttribute('action-id'),
                        index: commandSelector.selectedIndex-1,
                        value: valueInput.value
                    });
            }
        }
    }
}

function onSelectValue(valueSelector, data) {
    const row = $(valueSelector).closest('tr')[0];
    const targetSelector = $(row).find('.target')[0];
    const commandSelector = $(row).find('.command')[0];

    if (data !== undefined) {
        valueSelector.selectedIndex = data.value+1;
        row.setAttribute('action-value-index', data.value+1);
    }
    else if (commandSelector.classList.contains('control')) {
        $(valueSelector).closest('tr').find('button[name=commit]')[0].onclick = function () {
            $('#panel-table')
                .find('.select-row input:checkbox:checked')
                .closest('.action[action-target=' + targetSelector.value + ']')
                .each((i, r) => {
                    onSelectCommand($(r).find('.command').val(commandSelector.value)[0]);
                    onSelectValue($(r).find('.value').val(valueSelector.value)[0]);
                });
        };
    } else {
        request(SET_ACTION,
            function (response) {
                row.setAttribute('action-id', response);
                row.setAttribute('action-value-index', valueSelector.selectedIndex-1);
            },
            function (error, data) {
                valueSelector.selectedIndex = row.getAttribute('action-value-index');
                shake(valueSelector);
            },
            {
                eId: row.getAttribute('entity-id'),
                target: targetSelector.value,
                trigger: $(row).closest('table').closest('tr')[0].getAttribute('trigger-id'),
                action: row.getAttribute('action-id'),
                index: commandSelector.selectedIndex-1,
                value: valueSelector.value
            });
    }
}

function destroy(element) {
    $(element)
        .css('opacity', 0)
        .one('transitionend', () => $(element).closest('tr').remove());
}

function hide(element) {
    $(element)
        .css('opacity', 0)
        .one('transitionend', function () {
            $(element).addClass('hidden');
        });
}

function show(element) {
    $(element).removeClass('hidden');
    setTimeout(function () {
        $(element).css('opacity', 1);
    }, 100);
}

function onResize() {
    var zoom = document.body.clientWidth/1250;
    if (zoom > 0.9) {
        zoom = 0.9;
    }
    $(".parent").css({zoom: zoom});
}

function onTimeChanged(e) {
    const triggerTime = $(e.form).closest(' tr')[0].getAttribute('trigger-time');
    if (e.value) {
        $(e.form).find('input[name=commit]')[0].hidden = e.value === triggerTime;
    }
}

function commitTime(e) {
    const row = $(e.form).closest(' tr')[0];
    e.hidden = true;

    request(SET_TIME,
        function (response) {
            row.setAttribute('trigger-id', response.id);
            row.setAttribute('trigger-time', e.form.time.value);
        },
        function () {
            const timeInput = $(e.form).find('input[type=time]')[0];
            timeInput.value = row.getAttribute('trigger-time');
            shake(timeInput);
        },
        {
            location: locationId,
            trigger: row.getAttribute('trigger-id'),
            type: document.getElementsByClassName('panel-selected-td')[0].id,
            time: e.form.time.value
        }
    );
}