from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from .enginery import Enginery, _EngineryFunit


class Light(Enginery):
    def __init__(self, p_id, data):
        super(Light, self).__init__(p_id, data)
        self.power = data.get('attributes', {}).get('power', 0)

    def calc_on(self, variable):
        return None if variable.invalid else variable.value > 0

    def on_state(self, topic: State, variable: Variable):
        super(Light, self).on_state(topic, variable)

        if topic.entity_address.funit_type == BrightnessLevel:
            self.publish_state(On, self.calc_on(variable))

            for _subginery in self._subgineries:
                _subginery.on_level_changed(
                    eng=self,
                    level=None if variable.invalid else variable.value,
                    preliminary=variable.preliminary
                )

            if not variable.preliminary:
                if variable.invalid:
                    power = None
                else:
                    power = int(self.power * variable.value / 100)

                self.publish_state(Power, power, delay=Power.delay)


class SwitchingLight(Light):
    def calc_on(self, variable):
        return None if variable.invalid else variable.value == 100


class DimmingLight(Light):
    pass


class TunableWhiteLight(Light):
    pass


class RgbLight(Light):
    pass


class DynamicLight(Light):
    pass


class _LightFunit(_EngineryFunit):
    entity_type = Light


Light.funit_class = _LightFunit


class On(_LightFunit):
    @property
    def set_value(self): return 100 if self._variable.value else 0

    def set(self): self.entity.publish_provider_command(BrightnessLevel, self.set_value, sig=self.signature)


class BrightnessLevel(_LightFunit):
    pass


class Power(_LightFunit):
    delay = 0
    def set(self): raise self.NotImplemented()


class GroupLevel(_LightFunit):
    pass


class Temperature(_LightFunit):
    entity_type = TunableWhiteLight


Light.settable_units = [On, BrightnessLevel, GroupLevel]
TunableWhiteLight.gettable_units = Light.settable_units + [Temperature]
