import json
import logging
import os

from spread_core.bam.handling.items.triggers import Holiday
from spread_core.errors.project_errors import InitError
from spread_core.tools import manager_interface
from spread_core.tools.settings import config, CALENDAR


class Handler(manager_interface.ManagerOfBroker):
    triggers_path = None

    def __init__(self, project_id, _mqttc):
        self.stopped = False
        super(Handler, self).__init__(_mqttc)
        self.project_id = project_id
        self.triggers = {}

        self.update_config()

    @property
    def sessionID(self): return self.__class__.__name__

    def parse(self, data):
        raise NotImplementedError()

    def update_config(self):
        has_calendar = config.get(CALENDAR, None)
        super(Handler, self).update_config()

        if config.get(CALENDAR, None) != has_calendar:
            Holiday.days()

    def start(self):
        triggers_path = self.triggers_path
        if os.path.isfile(triggers_path):
            with open(triggers_path, 'r') as file:
                data = file.read()
            if not data:
                data = '[]'
            try:
                data = json.loads(data)
            except BaseException as ex:
                raise InitError(f'Ошибка парсинга файла "{triggers_path}": {ex}')
            self.parse(data)
            self.on_ready()
        else:
            raise InitError(f'{triggers_path} не существует!')

    def on_ready(self):
        pass

    def on_message(self, topic, variable):
        pass

    def on_exit(self):
        super(Handler, self).on_exit()
        self.stopped = True
        if self.mqttc:
            try:
                self.mqttc.disconnect()
                self.mqttc.loop_stop(True)
            except BaseException as ex:
                logging.error(ex)
