import logging

from .. import const, actions, triggers
from ..errors import ObjectNotFound


class Location:
    def __init__(self, _id: int = None, _enabled: bool = False, _name='', _tasks=[]):
        self._id = _id
        self.enabled = _enabled
        self.name = _name
        self.triggers = {}

    def get_id(self) -> int: return self._id
    def set_id(self, _id: int): self._id = _id

    id = property(fget=get_id, fset=set_id, doc='id')

    def __str__(self):
        return f'{self.name if self.name else self.__class__.__name__}({self.id}){ "DISABLED" if not self.enabled else ""}'

    def __repr__(self):
        return self.__str__()

    def __hash__(self):
        return self.id

    def __eq__(self, other):
        return other.id == self.id

    def get_trigger(self, id: int):
        for trigger in self.triggers:
            if trigger.id == id:
                return trigger

        raise ObjectNotFound.trigger(id)

    def remove_trigger(self, id):
        for trigger in self.triggers:
            if trigger.id == id:
                self.triggers.pop(trigger)
                return

        raise ObjectNotFound.trigger(id)

    def get_action(self, trigger_id, action_id):
        trigger = self.get_trigger(trigger_id)
        for action in self.triggers[trigger]:
            if action.id == action_id:
                return action

        raise ObjectNotFound.action(action_id)

    def remove_action(self, trigger_id, action_id):
        trigger = self.get_trigger(trigger_id)
        for action in self.triggers[trigger]:
            if action.id == action_id:
                self.triggers[trigger].remove(action)
                return

        raise ObjectNotFound.action(action_id)

    @classmethod
    def deserialize(cls, data: dict):
        location = Location(_id=data[const.ID], _enabled=data[const.ENABLED])
        if const.NAME in data:
            location.name = data[const.NAME]
        if const.TRIGGER in data and isinstance(data[const.TRIGGER], list):
            for trigger_data in data[const.TRIGGER]:
                try:
                    trigger = triggers.of(trigger_data)
                    if isinstance(trigger, triggers.Holiday) and not triggers.Holiday.days():
                        # Пропускать тригеры для выходных, если выходных не предвидится =)
                        continue
                    location.triggers[trigger] = []
                    for action_data in trigger_data[const.ACTIONS]:
                        try:
                            action = actions.action_of_data(action_data)
                            location.triggers[trigger].append(action)
                        except BaseException as ex:
                            logging.warning(ex)
                except BaseException as ex:
                    logging.warning(f'Ошибка парсинга триггера "{trigger_data}": {ex}')

        return location

    def serialize(self):
        res = dict(
            id=self.id,
            enabled=self.enabled,
            name=self.name,
            triggers=[]
        )

        for _trigger, _actions in self.triggers.items():
            trigger_data = _trigger.serialize()
            trigger_data['actions'] = []
            for action in _actions:
                trigger_data['actions'].append(action.serialize())
            res['triggers'].append(trigger_data)

        return res
