import logging
import threading

import spread_core.tools.settings as settings
from spread_core.errors.project_errors import InitError
from . import actions, triggers, const
from .handler import Handler


class Scripter(Handler):
    def __init__(self, project_id, _mqttc):
        super(Scripter, self).__init__(project_id, _mqttc)
        self.stack = []
        self.values = {}
        self.step = threading.Event()
        self.exec_thread = threading.Thread(target=self.execution, name='ExecThread')
        self.exec_thread.start()
        self.triggers = {}

    def parse(self, data):
        for trigger_data in data:
            try:
                trigger = triggers.of(trigger_data)
                self.triggers[trigger] = []
                for action_data in trigger_data[const.ACTIONS]:
                    try:
                        action = actions.action_of_data(action_data)
                        self.triggers[trigger].append(action)
                    except BaseException as ex:
                        logging.warning(ex)
                    else:
                        if action.conditioned:
                            for condition in action.conditions:
                                self.subscribe(condition.topic, log=True)
                if len(self.triggers[trigger]) == 0:
                    self.triggers.pop(trigger)
                elif isinstance(trigger, triggers.BrokerTrigger):
                    self.subscribe(trigger.topic, log=True)
            except BaseException as ex:
                logging.warning(ex)

        if len(self.triggers) == 0:
            raise InitError(f'Файл "{self.triggers_path}" не содержит ни одного триггера!')

    def update_config(self):
        super(Scripter, self).update_config()
        self.triggers_path = settings.config[settings.TRIGGERS]

    def on_message(self, topic, variable):
        self.values[str(topic)] = variable.value
        for trigger in self.triggers:
            if isinstance(trigger, triggers.BrokerTrigger):
                if trigger.check(topic, variable):
                    for action in self.triggers[trigger]:
                        if isinstance(action, actions.Action):
                            self.stack.append((action, variable.value))
                            if not self.step.is_set():
                                self.step.set()

    def on_exit(self):
        super(Scripter, self).on_exit()
        self.step.set()

    def get_condition_value(self, key):
        if key in self.values:
            return self.values[key]

        return None

    def execution(self):
        while self.stopped is False:
            while len(self.stack) > 0:
                if self.stopped:
                    return
                try:
                    action, rec_value = self.stack.pop(0)
                    action.execute(self.publish, rec_value, self.get_condition_value, session_id=self.sessionID)
                except BaseException as ex:
                    logging.error(ex)
            self.step.clear()
            self.step.wait()
