import inspect

from spread_core.tools.signature import CommandSignature


class MultiCommand:
    def __init__(self, func, *args, **kwargs):
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self._sig = None

    def call(self):
        parameters = inspect.signature(self.func).parameters
        _args_len = len(parameters)
        if _args_len == 0:
            self.func()
        elif 'sig' in parameters:
            self.func(*self.args[:(_args_len - 1)], sig=self._sig, **self.kwargs)
        else:
            self.func(*self.args[:(_args_len - 1)], **self.kwargs)

    def set_signature(self, funit_type, key, session_id):
        self._sig = CommandSignature(funit_type, key=key, session_id=session_id)

    @property
    def session_id(self):
        return self._sig.session_id if self._sig else None

    @property
    def key(self):
        return self._sig.key if self._sig else None

    def __str__(self):
        return '{}(sig, {})'.format(self.func.__name__, ('{}'*len(self.args)).format(*self.args))

    def __repr__(self):
        return str(self)

