class InvalidDevice(BaseException):
    def __init__(self, entity):
        super(InvalidDevice, self).__init__('Запрос/команда для невалидного объекта: ' + str(entity))


class CoreError(BaseException):
    def __init__(self, message, **attributes):
        self.message = message
        self.attributes = attributes

    def __str__(self):
        return self.message + ('\n' + str(self.attributes) if len(self.attributes) > 0 else '')


class ProjectError(CoreError):
    pass


class JocketError(CoreError):
    pass


class EngineryError(CoreError):
    def __init__(self, message, **attributes):
        super(EngineryError, self).__init__('[{}]: '.format(self.__class__.__name__) + message, **attributes)


class CommandError(EngineryError):
    @classmethod
    def of_exec(cls, message):
        return cls(message)


class EngineryNotPresent(EngineryError):
    pass


class ProviderNotPresent(EngineryError):
    def __init__(self, _type, _id):
        super(ProviderNotPresent, self).__init__(f'{_type}({_id}) is not present')


class ResponseError(CoreError):
    pass


class ClassifierError(CoreError):
    @classmethod
    def of_funit(cls, funit_type: str, *owner):
        return cls(f'"{funit_type}" отсутствует в {cls.__tree(owner)}')

    @classmethod
    def of_action(cls, action: str, *owner):
        return cls(f'Action "{action}" отсутствует в {cls.__tree(owner)}')

    @classmethod
    def no_action_type(cls, action: str, *owner):
        return cls(f'Action "{action}"" не содержит типов для {cls.__tree(owner)}')

    @classmethod
    def action_of_func(cls, func: str, *owner):
        return cls(f'Метод "{func}"" не определён в {cls.__tree(owner)}')

    @classmethod
    def of_parameter(cls, *parameter: str):
        return cls(f'Ошибка параметра для "{parameter}"')

    @staticmethod
    def __tree(tree):
        return "-".join(["classifier"] + [i if isinstance(i, str) else i.__class__.__name__ for i in tree])


class AddressError(BaseException):
    pass


class TopicError(BaseException):
    pass


class InitError(CoreError):
    pass


class AuthError(BaseException):
    CONNECT_RESULTS = [
        'Connection successful',
        'Connection refused - incorrect protocol version',
        'Connection refused - invalid client identifier',
        'Connection refused - server unavailable',
        'Connection refused - bad username or password',
        'Connection refused - not authorised',
    ]

    @staticmethod
    def connection_result(rc):
        try:
            return AuthError.CONNECT_RESULTS[rc]
        except IndexError:
            return 'Connect result is %s' % rc

    def __init__(self, rc):
        super(AuthError, self).__init__(self.connection_result(rc))
