import logging

from spread_core import mqtt
from spread_core.bam.handling.cycler import Cycler
from spread_core.errors.project_errors import AddressError, TopicError
from spread_core.tools.service_launcher import Launcher


class CyclerLauncher(Launcher):
    _dumped = False

    @property
    def manager(self):
        if self._manager is None:
            self._manager = Cycler(self.project_id, self.mqttc)
        return self._manager

    def on_message(self, mosq, obj, msg):
        super(CyclerLauncher, self).on_message(mosq, obj, msg)
        try:
            topic = mqtt.of(msg.topic)
            if isinstance(topic, mqtt.spread.topic.SpreadTopic):
                self._manager.on_spread(topic, msg.payload)
            elif isinstance(topic, mqtt.local.CycleTrigger):
                self._manager.on_trigger(topic, msg.payload)
            elif isinstance(topic, mqtt.local.Cycles):
                self._manager.on_cycle(topic, msg.payload)
            elif isinstance(topic, mqtt.project.TopicProject):
                self._manager.on_project(topic, msg.payload)

        except (AddressError, TopicError) as ex:
            logging.warning(ex)
        except BaseException as ex:
            logging.exception(ex)


def run():
    CyclerLauncher().connect()


if __name__ == '__main__':
    run()
