import spread_core.mqtt as mqtt
from spread_core.bam.knx.manager import Knx
from spread_core.errors import project_errors as err
from spread_core.tools.service_launcher import Launcher
from spread_core.tools.settings import *


class KnxManagerLauncher(Launcher):
    @property
    def manager(self):
        if self._manager is None:
            self._manager = Knx(self.project_id, self.manager_id)
            self._manager.set_broker(self.mqttc)
        return self._manager

    def on_message(self, mosq, obj, msg):
        super(KnxManagerLauncher, self).on_message(mosq, obj, msg)
        try:
            topic = mqtt.of(msg.topic)
            payload = msg.payload
            if isinstance(topic, mqtt.bus.TopicBus) and topic.is_dump:
                self._manager.send_interface.on_dump(payload)
            elif isinstance(topic, (mqtt.spread.topic.Set, mqtt.spread.topic.Renew)):
                self._manager.on_command(topic, mqtt.spread.variable.Variable.of(payload))
            elif isinstance(topic, mqtt.spread.topic.State):
                self._manager.on_state(topic, mqtt.spread.variable.Variable.of(payload))
            elif isinstance(topic, mqtt.project.TopicProject):
                self._manager.on_project(topic=topic,  data=payload)
            else:
                logging.debug('[{}] => {}'.format(msg.topic, payload.decode()))
        except err.ClassifierError as ex:
            logging.error(ex)
        except (err.ProjectError, err.InitError) as ex:
            logging.warning(ex)
        except BaseException as ex:
            logging.exception(ex)


def run():
    KnxManagerLauncher().connect()

# def run():
#     loop = asyncio.get_event_loop()
#     manager = KnxManagerLauncher(loop)
#     try:
#         loop.run_until_complete(manager.start())
#         loop.run_forever()
#     except (AuthError, ConnectionRefusedError) as ex:
#         logging.error(ex)
#     except KeyboardInterrupt:
#         logging.info('%s breaked' % manager.__class__.__name__)
#     finally:
#         loop.stop()


if __name__ == '__main__':
    run()
