import json
import logging

import spread_core.mqtt as mqtt
from spread_core.bam.lom.manager import Lom
from spread_core.errors.project_errors import CoreError, InitError
from spread_core.tools.service_launcher import Launcher


class LomManagerLauncher(Launcher):

    @property
    def manager(self):
        if self._manager is None:
            self._manager = Lom(self.project_id, self.manager_id)
            self._manager.set_broker(self.mqttc)
        return self._manager

    def on_message(self, mosq, obj, msg):
        super(LomManagerLauncher, self).on_message(mosq, obj, msg)
        try:
            payload = msg.payload.decode()
            topic = mqtt.of(msg.topic)
            if isinstance(topic, (mqtt.spread.topic.Set, mqtt.spread.topic.Renew)):
                self._manager.on_command(topic, mqtt.spread.variable.Variable.of(json.loads(payload)))
            elif isinstance(topic, mqtt.project.TopicProject):
                self._manager.on_project(topic=topic, data=payload)
            elif isinstance(topic, mqtt.spread.topic.State):
                self._manager.on_state(topic, mqtt.spread.variable.Variable.of(payload))
            elif isinstance(topic, mqtt.local.LomProfiles):
                self._manager.profile_changed(topic, payload)
            else:
                logging.debug('[{}] => {}'.format(msg.topic, payload))
        except InitError as ex:
            logging.error(ex)
            self.on_exit(0, 0)
        except CoreError as ex:
            logging.error(ex)
        except BaseException as ex:
            logging.exception(ex)


def run():
    LomManagerLauncher().connect()


if __name__ == '__main__':
    run()
