import logging

import spread_core.mqtt as mqtt
from spread_core.bam.mercury.managers import Mercury
from spread_core.errors.project_errors import InitError
from spread_core.tools.service_launcher import Launcher


class MercuryManagerLauncher(Launcher):
    _dumped = False

    @property
    def manager(self):
        if self._manager is None:
            self._manager = Mercury(self.project_id, self.manager_id)
            self._manager.set_broker(self.mqttc)
        return self._manager

    def on_message(self, mosq, obj, msg):
        super(MercuryManagerLauncher, self).on_message(mosq, obj, msg)

        try:
            topic = mqtt.of(msg.topic)
            assert topic
            if isinstance(topic, mqtt.bus.TopicBus):
                self._manager.send_interface.on_response(msg.payload.decode())
            elif isinstance(topic, (mqtt.spread.topic.Set, mqtt.spread.topic.Renew)):
                self._manager.on_command(topic, mqtt.spread.variable.Variable.of(msg.payload))
            elif isinstance(topic, mqtt.project.TopicProject):
                self._manager.on_project(topic, msg.payload.decode())
        except InitError as ex:
            logging.warning(ex)
        except BaseException as ex:
            logging.exception(ex)


def run():
    MercuryManagerLauncher().connect()


if __name__ == '__main__':
    run()
