import json

from datetime import datetime

ADDR = 'address'
ID = 'id'
CLASS = 'class'
TIMESTAMP = 'timestamp'
FLAGS = 'flags'
INVALID = 'invalid'
VALUE = 'value'
KEY = 'key'
DATA = 'data'
ERROR = 'error'
ACTION = 'action'
GET = 'get'


class VariableJocket:
    key = None
    action = None

    def __init__(self, data):
        self.timeStamp = None
        self.invalid = False
        self.key = None

        self.data = data
        self.kwargs = {}
        self.id = data[ADDR][ID]
        self.cl = data[ADDR].get(CLASS, 0)
        if TIMESTAMP in data:
            self.timeStamp = data[TIMESTAMP]
        if KEY in data:
            self.key = data[KEY]
        else:
            self.key = '{00000000-0000-0000-0000-000000000000}'
        if ACTION in data:
            self.action = data[ACTION]
        self.invalid = FLAGS in data and INVALID in data[FLAGS]
        # if not self.invalid:
        if DATA in data and data[DATA] is not None:
            for key, value in data[DATA].items():
                if key == VALUE:
                    self._value = value
                else:
                    self.kwargs[key] = value

    @property
    def value(self):
        return self.data.get(DATA, {}).get(VALUE, None)

    def set_value(self, value):
        self._value = value

    @value.setter
    def value(self, value):
        self._value = value
        if DATA not in self.data:
            self.data[DATA] = {}
        self.data[DATA][VALUE] = value

    def __repr__(self):
        return str(self)

    def __str__(self):
        if self.action == GET:
            return str.format('class: {}, id: {}, GET', self.cl, self.id)
        elif ERROR in self.data:
            return str.format('class: {}, id: {}, error: {}', self.cl, self.id, self.data['error'])
        else:
            r = ' '.join([str(self._value)] + [f'{param_key}={param_value}' for param_key, param_value in self.kwargs.items()])
            if self.invalid:
                r += '(Inv)'
            return r

    def pack(self):
        data = self.data
        if CLASS[:-1] in data[ADDR]:
            data[ADDR][CLASS] = data[ADDR].pop(CLASS[:-1])
        if self.invalid:
            if FLAGS not in data:
                data[FLAGS] = []
            if INVALID not in data[FLAGS]:
                data[FLAGS].append(INVALID)
        data[KEY] = self.key
        data[ACTION] = self.action
        return json.dumps(data)

    @staticmethod
    def create_data(id, cl, action=GET, val=None, key=None, invalid=False, **params):
        # if val is None or val == '':
        #     print('AHTUNG on create_data')
        data = dict()
        data[ADDR] = dict()
        data[ADDR][ID] = id
        data[ADDR][CLASS] = cl
        if key:
            data[KEY] = str(key)
        else:
            data[KEY] = '{00000000-0000-0000-0000-000000000000}'
        data[ACTION] = action
        data[TIMESTAMP] = str(datetime.now()).replace(' ', 'T')
        data[DATA] = params
        data[DATA][VALUE] = val
        if invalid:
            if FLAGS not in data:
                data[FLAGS] = []
            if INVALID not in data[FLAGS]:
                data[FLAGS].append(INVALID)
        return VariableJocket(data)
