import os
import re

from spread_core.errors.project_errors import TopicError
from .. import Topic


class Local(Topic):
    p_protocol = 'Project'
    protocol = 'Local'
    type = ''

    def __init__(self, project_id, entity_id: int):
        self.project_id = project_id
        try:
            self.entity_id = int(entity_id)
        except ValueError:
            self.entity_id = entity_id

    def __str__(self):
        return '/'.join((self.p_protocol, str(self.project_id), self.protocol, self.type, str(self.entity_id)))

    @classmethod
    def of(cls, topic: str):
        if re.search(r'%s/\d+/%s' % (cls.p_protocol, cls.protocol), topic):
            search = re.search(r'%s/(\d+)/%s/.+/(\d+)$' % (cls.p_protocol, cls.protocol), topic)
            if search:
                p_id, e_id = search.groups()
                if LAButtonPresets.type in topic:
                    return LAButtonPresets(p_id, e_id)
                elif LomProfiles.type in topic:
                    return LomProfiles(p_id, e_id)
                elif Cycles.type in topic:
                    if CycleTrigger.type in topic:
                        return CycleTrigger(p_id, topic[:topic.index(CycleTrigger.type) - 1].split('/')[-1], e_id)
                    return Cycles(p_id, e_id)
        raise TopicError(f'Unsupported topic "{topic}"')

    @classmethod
    def of_file(cls, project_id, dest, file):
        return '/'.join((cls.p_protocol, str(project_id), cls.protocol, dest[1:] if dest.startswith(os.path.sep) else dest, file))

    @classmethod
    def broadcast(cls, project_id):
        return '/'.join((cls.p_protocol, str(project_id), cls.protocol, cls.type, '#')).replace('//', '/')

    @classmethod
    def empty(cls, project_id):
        return '/'.join((cls.p_protocol, str(project_id), cls.protocol, cls.type, '')).replace('//', '/')


class LAButtonPresets(Local):
    type = 'Equipment/Lighting/LightingArea/ButtonPresets'


class LomProfiles(Local):
    type = 'Hardware/Lom/Profiles'


class Cycles(Local):
    type = 'Incubator/Cycles'


class CycleTrigger(Local):
    type = 'Triggers'

    def __init__(self, project_id: int, cycle_id, trigger_id):
        self.type = '/'.join((Cycles.type, str(cycle_id), CycleTrigger.type))
        self._cycle_id = cycle_id
        super(CycleTrigger, self).__init__(project_id=project_id, entity_id=trigger_id)

    @property
    def cycle_id(self):
        return try_get_int(self._cycle_id)


def try_get_int(obj):
    try:
        return int(obj)
    except:
        return str(obj)
