import can

from . import CONTROLLER, TYPES
from ..utils import bytes2str


class CanPacket:
    pr = 0

    def __init__(self, from_id, to_id, module_type, end_frame, cmd):
        self.from_id = from_id
        self.to_id = to_id
        self.module_type = module_type
        self.end_frame = end_frame
        self.cmd = cmd

    def __str__(self):
        return bytes2str(self.frame)

    def __repr__(self):
        return f'[{self.__class__.__name__}]: {self.__str__()}'

    @property
    def module_address(self):
        if self.from_id == CONTROLLER:
            return self.to_id
        else:
            return self.from_id

    @property
    def channel(self):
        return None

    @property
    def arbitration_id(self):
        return (self.pr << 10) + (self.from_id << 5) + self.to_id

    @property
    def module_desc(self):
        tp = self.module_type
        if tp in TYPES:
            return TYPES[tp]['desc']
        return f'Unknown module type "{hex(tp)}"'

    @property
    def module_name(self):
        tp = self.module_type
        if tp in TYPES:
            return TYPES[tp]['name']
        return f'Unknown module type "{hex(tp)}"'

    @property
    def frame(self) -> bytearray:
        raise NotImplementedError()

    @property
    def data(self) -> bytearray:
        raise NotImplementedError()

    @property
    def payload(self):# -> Payload:
        raise NotImplementedError()


class CanSend(CanPacket):
    BUS_TYPE = 'socketcan'
    commit_timeout = 2
    answer_timeout = 0.5

    def __init__(self, module_type, module_address, end_frame, cmd=0, **kwargs):
        super(CanSend, self).__init__(CONTROLLER, module_address, module_type, end_frame, cmd)
        self.need_answer = False
        self.send_twice = False


class CanDump(CanPacket):
    def __init__(self, from_id, to_id, module_type, frame, cmd=0):
        super(CanDump, self).__init__(from_id, to_id, module_type, frame, cmd)

    @property
    def payload(self):
        raise NotImplementedError()

    @property
    def frame(self):
        return self.end_frame

    @staticmethod
    def can_dump(message: can.Message) -> CanPacket:
        from_id = (message.arbitration_id >> 5) & int('11111', 2)
        to_id = message.arbitration_id & int('11111', 2)
        frame = message.data
        b0 = frame[0]
        cmd = b0 >> 7
        cl = b0 & int('11111', 2)

        if cmd == 0:
            if cl == 1:
                if from_id == CONTROLLER:
                    from .dali_packet import DaliSend
                    channel = DaliSend.channel_from_frame(frame)
                    b1 = frame[1]
                    dump = DaliSend(cl, to_id, channel, frame[3:])
                    dump.send_twice = b1 & 0x01
                    dump.need_answer = b1 & 0x04
                    return dump
                else:
                    """ CMD = 0 """
                    from .dali_packet import DaliDump
                    return DaliDump(from_id, to_id, cl, frame, cmd)
        else:
            """ CMD = 1"""
            from .rapida_packet import RapidaDump
            return RapidaDump(from_id, to_id, cl, frame, cmd)
