from spread_core.protocols import utils


class Command:
    _params = b''
    _opcode = None
    _query = False

    def __init__(self, *args):
        self._params = bytes([int(p) for p in args])

    def __str__(self):
        return utils.bytes2str(self.frame)

    @property
    def frame(self):
        return bytes([self._opcode]) + self.params

    @property
    def is_query(self):
        return self._query

    @property
    def params(self):
        return self._params


def of(frame, opcode=None) -> Command:
    if opcode is None:
        opcode, *frame = frame

    for sc in Command.__subclasses__() + Query.__subclasses__():
        if issubclass(sc, Command):
            if sc._opcode == opcode:
                return sc(*frame)

    return Command(*frame)


class Query(Command):
    _query = True


class QueryVersion(Query):
    _opcode = 0x00


class QueryUidMax(Query):
    _opcode = 0x04


class QueryUidMin(Query):
    _opcode = 0x06


class SetAddress(Command):
    _opcode = 0x10

    @property
    def params(self):
        if int(self._params[0]) not in range(0, 31):
            raise
        return super(SetAddress, self).params


class Reboot(Command):
    _opcode = 0xff


"""loader"""


class QueryBootLoaderInfo(Query):
    _opcode = 0xb0


class InitLoad(Query):
    _opcode = 0xb2


"""Addressing"""


class QueryAddressInfo(Query):
    _opcode = 0xa0


class RunAutoAddressing(Query):
    _opcode = 0xaa


class NextAddress(Query):
    _opcode = 0xac

    @property
    def params(self):
        if int(self._params[0]) not in range(0, 31):
            raise
        return super().params


class CommitAddresses(Query):
    _opcode = 0xad


"""DALI"""


class QueryLineStatus(Query):
    _opcode = 0x20


"""MODBUS"""


class SetChannelConfig(Command):
    _opcode = 0x40

    def __init__(self, channel, baud, parity, stop):
        """
        Настройка канала
        :param channel: номер канала (1-4)
        :param baud: скорость приема/передачи данных. Принимает значения:   0 - 1200
                                                                            1 - 2400
                                                                            2 - 4800
                                                                            3 - 9600
                                                                            4 - 19200
                                                                            5 - 38400
                                                                            6 - 57600
                                                                            7 - 115200
                                                                            8 - 230400
                                                                            9 - 460800
                                                                            10 - 921600


        :param parity - контроль четности. Принимает значения:  0 - none
                                                                1 - even
                                                                2 - odd
        :param stop - число стоп-битов. Принимает значения: 0 - 1 stop
                                                            1 - 2 stops
        """
        super(SetChannelConfig, self).__init__(*(channel, baud, parity, stop))
        self.channel = channel
        self.baud = baud
        self.parity = parity
        self.stop = stop


class ResetChannel(Command):
    """ Сброс настройки канала. Значение по умолчанию - 19200, EVEN, 1 STOP """
    _opcode = 0x41


class QueryChannelConfig(Query):
    _opcode = 0x42

    def __init__(self, channel):
        """
        Запрос настроек канала
        :param channel: номер канала
        """
        super(QueryChannelConfig, self).__init__(*(channel,))
        self.channel = channel
