import json
import os

import yaml

TEMP = dict(
    PROJECT_ID='projectID',
    MANAGER_ID='managerID',
    COMMUNICATE_TIMEOUT='communicateTimeout',
    TIMEOUT='timeout',
    LOG_SUBSCRIBES='logSubscribes',
    RESTART_TIMEOUT='restartTimeout',
    CTP_PATH='ctpPath',
    DUMP='dump',
    LOG_LEVEL='logLevel',
    DEBUGGING_HOST='debuggingHost',
    CALENDAR='calendar',
    TASKS='tasks',
    TRIGGERS='triggers',
    PROFILES='profiles',
    TEMP_DIMMING_PERIOD='tempDimmingPeriod',
    HTTP_ROOT='httpRoot',
    DUMP_TIMEOUT='dumpTimeout'
)


def check(file, handler, ext='.yaml'):
    print(file.center(80, '*'))
    for root, _, files in os.walk(file):
        for file in files:
            if file.endswith(ext):
                try:
                    file = os.path.join(root, file)
                    log = handler(file)
                    if log:
                        print('| OK ' + file)
                    else:
                        continue
                except BaseException as ex:
                    print(f'| error {file}: {ex}')
    print('*'*80)
    print()


def convert_dump(file):
    if os.path.exists(file.replace('.yaml', '.json')):
        return False
    data = yaml.load(open(file, "r"), Loader=yaml.FullLoader)
    with open(file.replace('.yaml', '.json'), 'w') as outstream:
        outstream.write(json.dumps(data, indent=2))

    return True


def convert_supervisor(file):
    with open(file, "r") as instream:
        data = instream.read()
    if '.yaml' in data:
        data = data.replace('.yaml', '.json')
        with open(file, 'w') as outstream:
            outstream.write(data)

        return True

    return False


def convert_conf(file):
    if os.path.exists(file.replace('.yaml', '.json')):
        return False
    fr = yaml.load(open(file, "r"), Loader=yaml.FullLoader)
    to = {}
    for key, value in fr.items():
        if key == 'include':
            value = [item.replace('.yaml', '.json') for item in value]
        elif key == 'MANAGER_TYPE':
            continue
        elif key == 'BROKER_HOST':
            to['broker'] = dict(host=fr['BROKER_HOST'], port=fr['BROKER_PORT'], login=fr['BROKER_USERNAME'], password=fr['BROKER_PASSWORD'])
            continue
        elif key in ['BROKER_PORT', 'BROKER_USERNAME', 'BROKER_PASSWORD']:
            continue
        elif key == 'BUS_TIMEOUT':
            to['bus'] = dict(id=fr['BUS_ID'], host=fr['BUS_HOST'], port=fr['BUS_PORT'], timeout=fr['BUS_TIMEOUT'], killTimeout=fr['KILL_TIMEOUT'])
            continue
        elif key in ['BUS_ID', 'BUS_HOST', 'BUS_PORT', 'KILL_TIMEOUT']:
            continue
        elif isinstance(value, str) and ".yaml" in value:
            if key in TEMP:
                key = TEMP[key]
            value = value.replace('.yaml', '.json')
        else:
            if key in TEMP:
                key = TEMP[key]

        to[key] = value

    with open(file.replace('.yaml', '.json'), 'w') as outstream:
        outstream.write(json.dumps(to, indent=2))

    return True


if __name__ == '__main__':
    check('/var/project/dump', convert_dump)
    check('/etc/supervisor/conf.d', convert_supervisor, ext='.conf')
    check('/etc/spread', convert_conf)
