import argparse
import json
import os
import shutil

SV_CONF = '/etc/supervisor/conf.d/'
SPREAD_CONF = '/etc/spread/'
SPREAD_DATA = '/var/project/'
DB_CONF = """[fcgi-program:spread_dashboard]
command=daphne -u /run/spread_dashboard.sock --fd 0 --access-log - --proxy-headers spread_dashboard.asgi:application
;process_name=%(program_name)s_%(process_num)02d
numprocs=1
directory=/var/spread
;umask=022
;priority=999
autostart=true
autorestart=true
startsecs=3
startretries=3
;exitcodes=0,2
;stopsignal=TERM
stopsignal=QUIT
stopwaitsecs=10
;stopasgroup=false
;killasgroup=false
user=root
redirect_stderr=false
stdout_logfile=/var/log/spread/dashboard_stdout.log
stdout_logfile_maxbytes=3MB
stdout_logfile_backups=10
;stdout_capture_maxbytes=1MB
;stdout_events_enabled=false
stderr_logfile=/var/log/spread/dashboard_stderr.log
stderr_logfile_maxbytes=1MB
stderr_logfile_backups=10
;stderr_capture_maxbytes=1MB
;stderr_events_enabled=false
environment=DJANGO_SETTINGS_MODULE=spread_dashboard.settings;SETTINGS={settings}
;serverurl=AUTO
socket=tcp://localhost:55582
;socket_owner=root
;socket_mode=0700
"""

DASHBOARD_STATIC_NAME = 'dashboard_static.json'
DASHBOARD_STATIC = {
    'httpRoot': 'spread'
}


def run():
    parser = argparse.ArgumentParser(description='Reset spread environment and all projects data')
    parser.add_argument('-y', '--yes', dest='commit', action='store_true', help='Don`t ask commit')
    args = parser.parse_args()

    if not args.commit and input('Are you sure? (y/N)> ').lower() not in ('y', 'yes', 'da'):
        print('Break')
        return

    for root, _, files in os.walk(SV_CONF):
        for file in files:
            if file.startswith('spread'):
                os.remove(os.path.join(root, file))
                print('removing ' + os.path.join(root, file))

    shutil.rmtree(SPREAD_DATA, ignore_errors=True)
    shutil.rmtree(SPREAD_CONF, ignore_errors=True)

    if not os.path.isdir(SV_CONF):
        os.mkdir(SV_CONF)
    with open(os.path.join(SV_CONF, 'spread_dashboard.conf'), 'w') as fs:
        fs.write(DB_CONF.replace('{settings}', os.path.join(SPREAD_CONF, DASHBOARD_STATIC_NAME)))

    configure_dashboard()

    os.system('sudo supervisorctl reread')
    os.system('sudo supervisorctl restart spread_dashboard')
    os.system('sudo supervisorctl update')

    os.system('sudo service mosquitto stop')
    shutil.rmtree('/var/lib/mosquitto/', ignore_errors=True)
    os.system('sudo service mosquitto start')


def configure_dashboard():
    if not os.path.isdir(SV_CONF):
        os.mkdir(SV_CONF)

    with open(os.path.join(SV_CONF, 'spread_dashboard.conf'), 'w') as fs:
        fs.write(DB_CONF.replace('{settings}', os.path.join(SPREAD_CONF, DASHBOARD_STATIC_NAME)))

    if not os.path.isdir(SPREAD_CONF):
        os.mkdir(SPREAD_CONF)

    with open(os.path.join(SPREAD_CONF, DASHBOARD_STATIC_NAME), 'w') as fs:
        json.dump(DASHBOARD_STATIC, fs, indent=4)


if __name__ == '__main__':
    run()
