import argparse
import json
import os
import sqlite3

parser = argparse.ArgumentParser(description='Lite`s database converter')
parser.add_argument('db', help="Database path")
parser.add_argument('file', help="Result file")
args = parser.parse_args()


def run():
    dump = {}
    if os.path.exists(args.db) and os.path.isfile(args.db):
        with sqlite3.connect(args.db) as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT id, type, attributes FROM 'engineries'")
            engs = cursor.fetchall()
            cursor.execute("SELECT location_id, scene_index, enginery_id, enginery_data FROM 'scenes'")
            scenes = cursor.fetchall()

        for entity_id, entity_type, data in engs:
            data = json.loads(data)
            if entity_type not in dump:
                dump[entity_type] = {}
            if entity_id not in dump[entity_type]:
                dump[entity_type][entity_id] = {}

            for key, value in data.items():
                key = key[0].upper() + key[1:]
                dump[entity_type][entity_id][key] = value

        dump['Lighting'] = {}

        for location_id, scene_index, enginery_id, enginery_data in scenes:
            enginery_data = json.loads(enginery_data)
            scene_name = f'scene_{scene_index}'
            dump['Lighting'].setdefault(location_id, {})
            dump['Lighting'][location_id].setdefault(scene_name, {})
            dump['Lighting'][location_id][scene_name][enginery_id] = enginery_data['BrightnessLevel']['Value']

        with open(args.file, 'w') as outfile:
            outfile.write(json.dumps(dump, indent=2))
        print(f'Config file saved as "{args.file}"')

    else:
        print(f'Database "{args.db}" not exist!')


if __name__ == '__main__':
    run()
