import threading
import logging
from collections import deque
from time import time


class Coroutine(object):
    STOP = 1
    TIMEOUT = 2
    """Делает из функции сопрограмму на базе расширенного генератора.

    @Timer.Coroutine
    def hello(timeout, *args, **kwargs):
        yield Timer.set(Timer.Coroutine.current, timeout)
        # post your code here...

    """
    current = None

    def __init__(self, callable):
        self._callable = callable

    def __call__(self, *args, **kwargs):
        corogen = self._callable(*args, **kwargs)
        try:
            Coroutine.current = corogen
            next(corogen)
        except StopIteration:
            pass
        except BaseException as ex:
            logging.exception(ex)
        return corogen


class Interrupt(BaseException):
    pass


class _DMeta(type):
    _instance = None

    def kill(cls):
        if cls._instance:
            cls._instance.clear()

        cls._instance = None

    @property
    def instance(cls):
        if cls._instance is None:
            cls._instance = Timer()
        return cls._instance

    @staticmethod
    @Coroutine
    def set(timeout: float, callback, *args, **kwargs):
        yield Timer.instance.push(Coroutine.current, timeout)
        if callable(callback):
            callback(*args, **kwargs)


class Timer(threading.Thread, metaclass=_DMeta):
    """Объект реализующий диспечер событий."""
    def __init__(self):
        super(Timer, self).__init__(name=self.__class__.__name__)
        self._pending = deque()
        self._deadline = None
        self.sleep = threading.Event()
        self.started = False
        self.update_deadline = False

    def clear(self):
        self._pending = deque()
        if not self.sleep.is_set():
            self.sleep.set()

    def push(self, corogen, timeout):
        deadline = time() + timeout
        self._pending.append([corogen, deadline])
        self._pending = deque(sorted(self._pending, key=lambda a: a[1]))

        if self._deadline is None or deadline < self._deadline:
            self._deadline = deadline
            self.update_deadline = True

            if not self.sleep.is_set():
                self.sleep.set()

        if not self.started:
            self.start()

    def run(self):
        """Запускает цикл обработки событий."""

        self.started = True
        while len(self._pending) > 0:

            timeout = self._deadline - time()

            while len(self._pending) > 0 and timeout > 0:
                self.sleep.clear()
                self.sleep.wait(timeout)
                timeout = self._deadline - time()

            while len(self._pending) > 0:
                if self._pending[0][1] <= time():
                    corogen, _ = self._pending.popleft()
                    if corogen:
                        try:
                            corogen.send(Coroutine.TIMEOUT)
                        except StopIteration:
                            pass
                        except BaseException as ex:
                            logging.exception(ex)
                else:
                    break
        self.__class__.kill()
