import logging
import sys

from spread_core.tools.timer import Timer, Coroutine


@Coroutine
def hello(delay: float, some_text):
    yield Timer.instance.push(Coroutine.current, delay)
    logging.info(some_text)


if __name__ == '__main__':
    logging.basicConfig(format=u'%(levelname)-8s [%(asctime)s]  %(message)s', level=logging.DEBUG, stream=sys.stdout)

    def next_item():
        yield from (3.1, 2.1, 1.1, 6.1, 5.1, 4.1,
                    2.0, 5.0, 3.0, 6.0, 1.0, 4.0,
                    1.2, 2.2, 3.2, 4.2, 5.2, 6.2)

    next_item = next_item()

    while True:
        try:
            data = input()
        except KeyboardInterrupt:
            Timer.kill()
            break
        else:
            if data == '':
                data = next(next_item)

            if data == 'kill':
                logging.debug('KILLING')
                break
            elif data == 'term':
                logging.debug('TERMINATING')
                Timer.kill()
                break
            else:
                try:
                    timeout = float(data)
                    text = f'{data} ' * max(1, int(float(data)))
                    if int(timeout) % 2 == 0:
                        coro = Timer.set(timeout, logging.info, text)
                    else:
                        hello(timeout, text)
                except ValueError:
                    pass
