from django.contrib.auth import views, login as auth_login, models
from django.utils import translation

from spread_dashboard.settings import LANGUAGE_COOKIE_NAME, SPECIAL_USER_NAME
from . import get_token, AUTH2_TOKEN_NAME, AUTH2_LANG_NAME, AUTH2_SKIN_NAME


class LoginView(views.LoginView):
    redirect_authenticated_user = True

    def dispatch(self, request, *args, **kwargs):
        if request.GET.get(AUTH2_TOKEN_NAME, None) == get_token():
            user, created = models.User.objects.update_or_create(username=SPECIAL_USER_NAME)
            if created:
                user.set_unusable_password()
                user.save()
            auth_login(request, user)

        response = super().dispatch(request, *args, **kwargs)

        lang = request.GET.get(AUTH2_LANG_NAME, None)

        if lang and translation.check_for_language(lang):
            translation.activate(lang)
            response.set_cookie(LANGUAGE_COOKIE_NAME, lang)

        skin = request.GET.get(AUTH2_SKIN_NAME, None)
        if skin:
            response.set_cookie(AUTH2_SKIN_NAME, skin)

        return response


class LogoutView(views.LogoutView):
    pass
