from django.contrib.auth.decorators import login_required
from django.shortcuts import render

from spread_core import mqtt
from spread_core.bam.subgineries import Incubator
from spread_core.tools.settings_const import CYCLES_FROM_0
from spread_dashboard.project import bam
from spread_dashboard.utils import app


@login_required
def locations(request):
    return render(
        request, 'incubator/locations.html',
        dict(
            broker=bam.broker_auth(),
            start_from_0=app().settings.get(CYCLES_FROM_0, False),
            locations=mqtt.project.TopicProject.locations(app().project_id),
            cycles=mqtt.local.Cycles.empty(app().project_id),
            states=mqtt.spread.topic.State(
                mqtt.spread.address.SubgineryAddress.broadcast(app().project_id, subginery_type=Incubator.__name__)
            ),
            start_index=0 if app().settings.get(CYCLES_FROM_0, False) else 1
        )
    )


@login_required
def cycles(request):
    return render(request, 'incubator/cycles.html', dict(
        broker=bam.broker_auth(),
        cycles=mqtt.local.Cycles.empty(app().project_id),
        triggers=mqtt.local.CycleTrigger(app().project_id, '+', ''),
        start_index=0 if app().settings.get(CYCLES_FROM_0, False) else 1
    ))


@login_required
def cycle(request, cid):
    return render(request, 'incubator/cycle.html', dict(
        start_index=0 if app().settings.get(CYCLES_FROM_0, False) else 1,
        broker=bam.broker_auth(),
        data=str(mqtt.local.Cycles(app().project_id, cid)),
        triggers=mqtt.local.CycleTrigger(app().project_id, cid, ''),
    ))
