import datetime
import sys

from django import http
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.utils.translation import gettext as _

from spread_core.bam.mercury.providers import MercuryElectricMeter
from spread_dashboard.project import bam
from spread_dashboard.responses import Success, Error


@login_required
def on_api(request, api_name):
    if hasattr(sys.modules[__name__], api_name):
        try:
            return getattr(sys.modules[__name__], api_name)(request)
        except BaseException as ex:
            print(str(ex))
            return http.HttpResponseServerError(str(ex))

    return http.HttpResponseNotAllowed(api_name)


@login_required
def page(request, meter_id=0):
    try:
        if meter_id:
            meter = bam.models.Provider.objects.get(id=meter_id, type=MercuryElectricMeter.__name__)
            stack = {meter.id: meter.name}
        else:
            stack = {id: name if name else 'id:%s' % id for (id, name) in
                     bam.models.Provider.objects.filter(type=MercuryElectricMeter.__name__).values_list('id', 'name')}
            if 'id' in request.GET:
                meter_id = int(request.GET.get('id'))
            elif len(stack) > 0:
                meter_id = list(stack.keys())[0]

            if meter_id:
                meter = bam.models.Provider.objects.get(id=meter_id, type=MercuryElectricMeter.__name__)
            else:
                meter = None

        if meter:
            ids = list(stack.keys())
            prev_id = ids[ids.index(meter.id) - 1]
            next_id = ids[(ids.index(meter.id) + 1) % len(stack)]
            now = datetime.datetime.now()

            MONTHS = [None, _('january'), _('february'), _('march'), _('april'),
                      _('may'), _('june'), _('july'), _('august'), _('september'),
                      _('october'), _('november'), _('december')]

            context = dict(
                config=bam.config(meter),
                meter=meter,
                stack=stack,
                prev_id=prev_id,
                next_id=next_id,
                energy=dict(
                    years={1: now.year, 2: now.year - 1},
                    monthes={**{MONTHS.index(month): f'{now.year} {month}' for month in MONTHS[now.month:0:-1]},
                             **{MONTHS.index(month): f'{now.year - 1} {month}' for month in MONTHS[now.month + 1:]}},
                    days={4: _('mercury_current_day'), 5: _('mercury_prev_day')}
                ),
            )

        else:
            context = dict(
                config=bam.config(meter),
            )

        return render(request, 'mercury/mercury.html', context=context)
    except bam.models.Manager.DoesNotExist as ex:
        return http.HttpResponse('<h1> ' + str(ex) + ' </h1>')


@login_required
def check_password(request):
    _id = int(request.GET.get('id'))
    _password = request.GET.get('password')
    model = bam.models.Provider.objects.get(id=_id, type=MercuryElectricMeter.__name__)
    if model.attributes['password2'] == _password:
        return Success()
    return Error(_('mercury_password_invalid'))
