import logging
import threading


class WSVendor(threading.Thread):
    def __init__(self):
        super(WSVendor, self).__init__(name=self.__class__.__name__)

        self.listeners = {}
        self.start()

    def run(self) -> None:
        raise NotImplementedError()

    def _on_connect(self):
        pass

    def _on_recv(self, topic, payload):
        for listener in self.listeners.values():
            listener.handler(topic, payload)

    def set_listener(self, group, handler):
        logging.debug('add group ' + group)
        try:
            self.listeners.setdefault(group, Listener(group, handler))
        except BaseException as ex:
            logging.exception(ex)

    def add_subscription(self, group, *topics):
        raise NotImplementedError()

    def remove_subscription(self, group, *topics):
        raise NotImplementedError()

    def remove_listener(self,  group):
        logging.debug('del group ' + group)
        if group in self.listeners:
            self.remove_subscription(group, *self.listeners[group].topics)
            self.listeners.pop(group)
        else:
            logging.debug(group, ' listener not set')


class Listener:
    def __init__(self, group, handler):
        self.group = group
        self.handler = handler
        self.topics = {}

    def compare(self, _topic):
        for topic in self.topics:
            if topic == _topic:
                return True
            elif '#' in topic:
                if _topic.index(topic[:topic.index('#')]) != -1:
                    return True

        return False
