import logging

from spread_dashboard.monitor.vendors.base_vendor import WSVendor
from spread_core.tools.settings_const import BROKER, LOGIN, PASSWORD, HOST, PORT
from spread_core.tools.mqtt_client import MqttClient
from spread_dashboard.utils import app


class WSVendorBroker(WSVendor):
    def __init__(self):
        self._mqttc = MqttClient(self,
                                 app().settings[BROKER][LOGIN],
                                 app().settings[BROKER][PASSWORD],
                                 self._on_connect,
                                 self._on_message)

        super(WSVendorBroker, self).__init__()

    def run(self) -> None:
        self._mqttc.connect(host=app().settings[BROKER][HOST],
                            port=app().settings[BROKER][PORT])
        self._mqttc.loop_forever()

    def _on_message(self, mosq, obj, msg):
        self._on_recv(msg.topic, msg.payload.decode())

    def add_subscription(self, group, *topics):
        for topic in topics:
            self.listeners[group].topics.setdefault(topic, 0)
            self.listeners[group].topics[topic] += 1
            if self.listeners[group].topics[topic] == 1:
                logging.debug(f'subscribe {topic}')
                self._mqttc.subscribe(topic)

    def remove_subscription(self, group, *topics):
        for topic in topics:
            if topic in self.listeners[group].topics:
                self.listeners[group].topics[topic] -= 1
                if self.listeners[group].topics[topic] == 0:
                    logging.debug(f'unsubscribe {topic}')
                    self._mqttc.unsubscribe(topic)
