from django.db import models
from jsonfield import JSONField


class Entity(models.Model):
    class Meta:
        abstract = True

    id = models.PositiveIntegerField(primary_key=True)
    type = models.CharField(max_length=255)
    name = models.CharField(max_length=255, blank=True)
    attributes = JSONField()
    key = models.CharField(max_length=64)

    @property
    def v_name(self): return self.name if self.name else 'id:%s' % self.id

    @property
    def name_not_star(self):
        if self.name.endswith('*'):
            return self.name[:-1]
        return self.v_name

    def serialize(self) -> dict:
        return {field.attname: getattr(self, field.attname) for field in self._meta.fields if not field.auto_created}

    @staticmethod
    def separator(entity: dict) -> dict: return entity

    # def __init__(self, *args, **kwargs):
    #     super(Entity, self).__init__(*args, **{k.attname: kwargs[k.attname] for k in self._meta.fields if k.attname in kwargs})


class Manager(Entity):
    serverID = models.PositiveIntegerField()


class Enginery(Entity):
    recipe = JSONField(blank=True)
    label = models.CharField(max_length=16)

    @staticmethod
    def get_scheduled(loc_id):
        return Enginery.objects.filter(
            name__endswith='*',
            id__in=Subginery.objects.get(
                id=loc_id,
                type='Lighting').engineries
        )


class Provider(Entity):
    managerID = models.PositiveIntegerField()


class Location(Entity):
    scheduled = models.BooleanField()
    navigations = JSONField(default=[], blank=True)
    models = JSONField(default=[], blank=True)
    type = None
    _active_engineries = None

    @staticmethod
    def separator(entity: dict) -> dict:
        scheduled = False
        if entity.get('name', '').endswith('*'):
            entity['name'] = entity.get('name', '')[:-1].strip()
            scheduled = True

        return {**{k: entity[k] for k in entity if k in ['id', 'name', 'key', 'navigations', 'models']}, 'scheduled': scheduled}

    @property
    def active_engineries(self):
        if self._active_engineries is None:
            self._active_engineries = []
            for model in self.models:
                for ctrl in model.get('controls', []):
                    if ctrl.get('clickable', False) and str(ctrl.get('engineryID', None)).isdigit():
                        self._active_engineries.append(int(ctrl['engineryID']))

        return self._active_engineries
                    
    def is_clickable(self, e_id):
        return e_id in self.active_engineries

    def __str__(self): return '%s(%s)' % (self.name, self.id)


class Subginery(Entity):
    locationID = models.PositiveIntegerField()
    engineries = JSONField(blank=True)
