from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import TemplateView

import spread_core.mqtt as mqtt
from spread_core.bam.engineries import light_types
from spread_dashboard.project import bam
from spread_dashboard.utils import app
from ..project.models import Location, Subginery, Enginery


class Report(LoginRequiredMixin, TemplateView):
    template_name = 'report/light.html'

    def get(self, request, *args, **kwargs):
        locations = {}
        for loc in Location.objects.all():
            node = locations
            for l_id in loc.navigations:
                node.setdefault('locations', [])
                cl = None
                for _cl in node['locations']:
                    if _cl.get('id') == l_id:
                        cl = _cl
                        break

                if cl is None:
                    cl = {'id': l_id}
                    node['locations'].append(cl)
                node = cl
                node.setdefault('locations', [])
                node.setdefault('engineries', [])

            try:
                node['id'] = loc.id
                node['name'] = loc.name
                node['engineries'] = [
                    {
                        'type': e.type,
                        'id': e.id,
                        'name': e.v_name,
                        'power': e.attributes.get('power', 0),
                        'renew': str(mqtt.spread.topic.Renew(
                            mqtt.spread.address.EngineryAddress(app().project_id, e.type, e.id, 'Chart')
                        )),
                    } for e in Enginery.objects.filter(
                        id__in=Subginery.objects.get(locationID=loc.id, type='Lighting').engineries
                    )
                    if e.type in light_types and loc.is_clickable(e.id)
                ]
            except Subginery.DoesNotExist:
                pass

        return self.render_to_response({
            'location': locations['locations'][0],
            'projectName': app().project.name,
            'broker': bam.broker_auth()
        })
