
const CYCLE = 0;
const DAY = 1;

class GraphFrame {
	x
	y;
	width;
	height;
}

// Отрисовка прямой
function DrawLine(ctx, fromX, fromY, toX, toY, lineWidth, strokeStyle) {
	ctx.strokeStyle = strokeStyle;
	ctx.lineWidth = lineWidth;
	ctx.beginPath();
	ctx.moveTo(fromX, fromY);
	ctx.lineTo(toX, toY);
	ctx.stroke();
	ctx.closePath();
}

// Вывод текста
function DrawText(ctx, x, y, textLine, fillStyle, font) {
	ctx.fillStyle = fillStyle;
	ctx.font = font;
	ctx.fillText(textLine, x, y);
}

// Вывод координатной горизонтальной прямой
function DrawCoordinateLine(ctx, graphFrame, headerHeight, y, textLine) {
	DrawLine(ctx,
		headerHeight, y,
		graphFrame.x + graphFrame.width, y,
		1, "#666666");
	DrawText(ctx, headerHeight / 2, y + headerHeight / 12, textLine, "#666666", "normal " + (headerHeight/4) + "px Arial");
}

// Отрисовка ломаной по массиву точек
function DrawBrokenLine(ctx, points, lineWidth, strokeStyle) {
	ctx.strokeStyle = strokeStyle;
	ctx.lineWidth = lineWidth;
	ctx.beginPath();
	ctx.moveTo(points[0][0], points[0][1]);
	for(var i = 1; i < points.length; i++) {
		// Проверяем, нет ли пересечения у ломаной из-за того, что следующая точка оказывается левее предыдущей
		if(i < points.length - 4) {
			if(points[i + 2][0] < points[i + 1][0]) { // Пересечение есть. У нас проблемы. Сейчас будем их порешать
				var x1 = points[i][0];
				var y1 = points[i][1];
				var x2 = points[i + 1][0];
				var y2 = points[i + 1][1];
				var x3 = points[i + 2][0];
				var y3 = points[i + 2][1];
				var x4 = points[i + 3][0];
				var y4 = points[i + 3][1];
				var n;

				if((y2 - y1) !== 0) {  // a(y)
       				var q = (x2 - x1) / (y1 - y2);
        			var sn = (x3 - x4) + (y3 - y4) * q; if (!sn) { return 0; }  // c(x) + c(y)*q
        			var fn = (x3 - x1) + (y3 - y1) * q;   // b(x) + b(y)*q
        			n = fn / sn;
    			}
    			else{
        			if (!(y3 - y4)) { return 0; }  // b(y)
        			n = (y3 - y1) / (y3 - y4);   // c(y)/b(y)
    			}
    			points[i + 2][0] = x3 + (x4 - x3) * n;  // x3 + (-b(x))*n
    			points[i + 2][1] = y3 + (y4 - y3) * n;  // y3 +(-b(y))*n

				ctx.lineTo(points[i][0], points[i][1]);
				ctx.stroke();

				i+=2;
			}
		}
		ctx.lineTo(points[i][0], points[i][1]);
		ctx.stroke();
	}
	ctx.closePath();
}

// Отрисовка ломаной графика по массиву значений. Рисуется внутри graphFrame. Начало координат - левый нижний угол graphFrame. Абсцисса от 0 до limitX. Ордината от 0 до limitY.
function DrawGraphBrokenLine(ctx, values, limitX, limitY, graphFrame, lineWidth, strokeStyle) {
	var points = [];
	for(var i = 0; i < values.length; i++) {
		points[i] = [];
		points[i][0] = graphFrame.x + values[i][0] * (graphFrame.width / limitX);
		points[i][1] = graphFrame.y + graphFrame.height - (values[i][1] * (graphFrame.height / limitY));
	}
	DrawBrokenLine(ctx, points, lineWidth, strokeStyle);
}

// Рассчет отступов от краев канвы т.е. определение области; вывод заголовка и подвала; вывод координатной сетки; вывод текущего дня и времени (если заданы)
function MakeFrame(graphCanvas, ctx, title, footer, frameType, numberOfDays, currentDay, currentTime) {
	var graphFrame = new GraphFrame();

	// Расчитаем высоту хидера
	var headerHeight = graphCanvas.height / 5;

	// Определим область под график, включая всю координатную сетку
	graphFrame.x = headerHeight / 2;
	graphFrame.y = headerHeight;
	graphFrame.width = graphCanvas.width - headerHeight;
	graphFrame.height = graphCanvas.height - 2 * headerHeight;

	// Вывод заголовка
	DrawText(ctx, headerHeight / 2, headerHeight / 2 + headerHeight / 6, title, "#AAAAAA", "normal " + (headerHeight/3) + "px Arial");

	// Вывод подвала
	DrawText(ctx, headerHeight, graphCanvas.height - headerHeight / 2.2, footer, "#AAAAAA", "normal " + (headerHeight/4) + "px Arial");

	/// Вывод координатной сетки по Y
	//0%
	DrawCoordinateLine(ctx, graphFrame, headerHeight, graphCanvas.height - headerHeight, "0%");
	//50%
	DrawCoordinateLine(ctx, graphFrame, headerHeight, headerHeight + graphFrame.height / 2, "50%");
	//100%
	DrawCoordinateLine(ctx, graphFrame, headerHeight, headerHeight, "100%");

	// Отрежем от области под график подписи горизонтальных линий
	// Т.о. останется область непосредственно самого графика
	graphFrame.x = headerHeight;
	graphFrame.width = graphFrame.width - headerHeight / 2;

	// Вывод координатной сетки по X
	if(frameType == CYCLE) {
		// График - весь цикл. Внизу сетки - дни цикла
		for(var i = 0; i <= numberOfDays; i++) {
			DrawLine(ctx,
				graphFrame.x + i * graphFrame.width / numberOfDays, graphFrame.y + graphFrame.height + headerHeight/20,
				graphFrame.x + i * graphFrame.width / numberOfDays, graphFrame.y + graphFrame.height + headerHeight/20 + headerHeight/4,
				1, "#666666");
			if(i < numberOfDays) {
				var d; // Это для правильного центрирования чисел разной длины
				if(i < 10) {d = headerHeight/20}
				else {d = headerHeight/10}

				if(i === currentDay ) { // Если задан текущий день и он равен тому дню, на котором мы сейчас находимся, выводим подпись по-другому
					ctx.beginPath();
					ctx.fillStyle = "#AAAAAA";
					ctx.rect(graphFrame.x + i * graphFrame.width / numberOfDays, graphFrame.y + graphFrame.height + headerHeight/20,
						graphFrame.width / numberOfDays, headerHeight/3.5);
					ctx.fill();

					DrawText(ctx,
						graphFrame.x + i * graphFrame.width / numberOfDays + graphFrame.width / numberOfDays / 2 - d,
						graphFrame.y + graphFrame.height + headerHeight/4,
						i + startIndex, "#000000", "normal " + (headerHeight/5) + "px Arial");
				} else {
					DrawText(ctx,
						graphFrame.x + i * graphFrame.width / numberOfDays + graphFrame.width / numberOfDays / 2 - d,
						graphFrame.y + graphFrame.height + headerHeight/4,
						i + startIndex, "#777777", "normal " + (headerHeight/5) + "px Arial");
				}
			}
		}
	} else {
		// График - один день цикла. Внизу сетки - часы
		for(var i = 0; i <= 24; i++) {
			DrawLine(ctx,
				graphFrame.x + i * graphFrame.width / 24, graphFrame.y + graphFrame.height + headerHeight/20,
				graphFrame.x + i * graphFrame.width / 24, graphFrame.y + graphFrame.height + headerHeight/20 + headerHeight/10,
				1, "#666666");

			var d; // Это для правильного центрирования чисел разной длины
			if(i < 10) {d = headerHeight/20}
			else {d = headerHeight/10}

			DrawText(ctx,
				graphFrame.x + i * graphFrame.width / 24 - d,
				graphFrame.y + graphFrame.height + headerHeight/3,
				i, "#777777", "normal " + (headerHeight/5) + "px Arial");
		}

		if(currentTime != null) { // Если задано текущее время в минутах с начала дня рисуем вертикальную черту на графике
			DrawLine(ctx,
				graphFrame.x + graphFrame.width * currentTime / (24 * 60), graphFrame.y,
				graphFrame.x + graphFrame.width * currentTime / (24 * 60), graphFrame.y + graphFrame.height,
				2, "#AAAAAA");
		}
	}

	return graphFrame;
}

// Переводим триггеры всего цикла в набор точек [время в мин от начала цикла][значения яркости],
// по которому будет строится ломаная графика
function ConvertTriggersToValues(sourceTriggers, startBrightness, fadeTime, numberOfDays) {
	let valuesCnt = 1;
	let prevBrightness = startBrightness;
	let absTime;
	let values = [];

	// Начальная точка графика
	values[0] = [];
	values[0][0] = 0;
	values[0][1] = startBrightness;

	for(var i = 0; i < sourceTriggers.length; i++) {
		absTime = sourceTriggers[i][0] * 1440 + sourceTriggers[i][1]; // Абсолютное время триггера от начала цикла
		// Горизонтальная линия
		values[valuesCnt] = [];
		values[valuesCnt][0] = absTime;
		values[valuesCnt][1] = prevBrightness;
		valuesCnt++;
		// Линия рассвета/заката
		values[valuesCnt] = [];
		values[valuesCnt][0] = absTime + fadeTime;
		values[valuesCnt][1] = sourceTriggers[i][2];
		prevBrightness = values[valuesCnt][1];
		valuesCnt++;
	}

	// Последнее значение яркости в цикле
	values[valuesCnt] = [];
	values[valuesCnt][0] = numberOfDays * 1440;
	values[valuesCnt][1] = prevBrightness;

	return values;
}

// Нормализуем исходные данные
function NormalizeTriggers(sourceTriggers, fadeTime, startBrightness) {
//for(let i = 0; i < triggers.length; i++) {alert(triggers[i][2]);}
	// Решаем проблему, когда освещение не успевает полностью измениться до начала следующего триггера и график рисуется криво
	// Записываем в триггеры те значения яркости, до которых успевает измениться свет, к моменту наступления следующего триггера
	let nextTriggerTime;
	let currentTriggerTime;
	let prevTriggerValue;

	// Делаем копию массива с триггерами
	var triggers = [];
	for(let i = 0; i < sourceTriggers.length; i++) {
		triggers[i] = [];
		triggers[i][0] = sourceTriggers[i][0];
		triggers[i][1] = sourceTriggers[i][1];
		triggers[i][2] = sourceTriggers[i][2];
	}

	for(let i = 0; i < triggers.length - 1; i++) {
		if(i === 0) {
			prevTriggerValue = startBrightness;
		} else {
			prevTriggerValue = triggers[i - 1][2];
		}
		nextTriggerTime = triggers[i + 1][0] * 1440 + triggers[i + 1][1];
		currentTriggerTime = triggers[i][0] * 1440 + triggers[i][1];
		if(fadeTime > nextTriggerTime - currentTriggerTime) {
			triggers[i][2] = prevTriggerValue + (triggers[i][2] - prevTriggerValue) * (nextTriggerTime - currentTriggerTime) / fadeTime;
		}
	}
//for(let i = 0; i < triggers.length; i++) {alert(triggers[i][2]);}
	return triggers;
}

// Нормализуем значения, по которым будет строится график
function NormalizeValues(values) {
	// Решаем проблему, когда освещение не успевает полностью измениться до начала следующего триггера и график рисуется криво
	// Удаляем лишние значения, которые появились из-за этой проблемы
	for(var i = 0; i < values.length - 1; i++) {
		if(values[i][0] >= values[i + 1][0]) {
			values.splice(i, 1);
		}
	}
	return values;
}

// Вывод на канву графика всего цикла
function DrawCycleGraph(graphCanvas, sourceTriggers, startBrightness, fadeTime, title, units, numberOfDays, currentDay, currentTime) {
	let ctx = graphCanvas.getContext('2d');
	ctx.clearRect(0, 0, graphCanvas.width, graphCanvas.height);
	let graphFrame = MakeFrame(graphCanvas, ctx, title, units, CYCLE, numberOfDays, currentDay, null);
	let values = [];

	// Нормализуем исходные данные
	let triggers = NormalizeTriggers(sourceTriggers, fadeTime, startBrightness);
	// Переводим триггеры всего цикла в набор точек
	values = ConvertTriggersToValues(triggers, startBrightness, fadeTime, numberOfDays);
	// Нормализуем значения, по которым будет строится график
	values = NormalizeValues(values);

	DrawGraphBrokenLine(ctx, values, numberOfDays * 1440, 100, graphFrame, 1, "#ff6101");
}

// Вывод на канву графика одного дня
function DrawDayGraph(graphCanvas, sourceTriggers, startBrightness, fadeTime, title, units, numberOfDays, currentDay, currentTime) {
	let ctx = graphCanvas.getContext('2d');
	ctx.clearRect(0, 0, graphCanvas.width, graphCanvas.height);
	let graphFrame = MakeFrame(graphCanvas, ctx, title, units, DAY, numberOfDays, currentDay, currentTime);
	let values = [];
	let dayValues = [];
	let valuesCnt = 0;
	let startDay = currentDay * 1440;
	let stopDay = (currentDay + 1) * 1440;

	// Нормализуем исходные данные
	let triggers = NormalizeTriggers(sourceTriggers, fadeTime, startBrightness);
	// Переводим триггеры всего цикла в набор точек
	values = ConvertTriggersToValues(triggers, startBrightness, fadeTime, numberOfDays);
	// Нормализуем значения, по которым будет строится график
	values = NormalizeValues(values);

	// Ищем точку для левого края графика
	for(let i = 0; i < values.length; i++) {
		if(values[i][0] === startDay) {
			// Есть точное совпадение с одной из точек всего цикла
			dayValues[0] = [];
			dayValues[0][0] = values[i][0];
			dayValues[0][1] = values[i][1];
			valuesCnt++;
		}
	}
	if(valuesCnt === 0) { // Точного совпадения не нашли. Ищем две точки, между которыми располагается левый край дневного графика и вычисляем значение освещенности в этой точке
		for(let i = 0; i < values.length && valuesCnt === 0; i++) {
			if(startDay < values[i][0]) {
				dayValues[0] = [];
				dayValues[0][0] = startDay;
				dayValues[0][1] = values[i - 1][1] + (values[i][1] - values[i - 1][1]) * (startDay - values[i - 1][0]) / (values[i][0] - values[i - 1][0]);
				valuesCnt++;
			}
		}
	}

	// Ищем точки, попадающие в график этого дня
	for(let i = 0; i < values.length; i++) {
		if (values[i][0] > startDay && values[i][0] < stopDay) {
			dayValues[valuesCnt] = [];
			dayValues[valuesCnt][0] = values[i][0];
			dayValues[valuesCnt][1] = values[i][1];
			valuesCnt++;
		}
	}

	// Ищем точку для правого края графика
	for(let i = 0; i < values.length; i++) {
		if(values[i][0] === stopDay) {
			// Есть точное совпадение с одной из точек всего цикла
			dayValues[valuesCnt] = [];
			dayValues[valuesCnt][0] = values[i][0];
			dayValues[valuesCnt][1] = values[i][1];
			valuesCnt = 0;
		}
	}
	if(valuesCnt !== 0) { // Точного совпадения не нашли. Ищем две точки, между которыми располагается левый край дневного графика и вычисляем значение освещенности в этой точке
		for(let i = 0; i < values.length && valuesCnt !== 0; i++) {
			if(stopDay < values[i][0]) {
				dayValues[valuesCnt] = [];
				dayValues[valuesCnt][0] = stopDay;
				dayValues[valuesCnt][1] = values[i - 1][1] + (values[i][1] - values[i - 1][1]) * (stopDay - values[i - 1][0]) / (values[i][0] - values[i - 1][0]);
				valuesCnt = 0;
			}
		}
	}

	for(let i = 0; i < dayValues.length; i++) {
		dayValues[i][0] = dayValues[i][0] - startDay; // Смещаем график в 0, иначе он не нарисуется
	}
	DrawGraphBrokenLine(ctx, dayValues, 1440, 100, graphFrame, 1, "#ff6101");
}
