var currentId;
var ids = [];

const generateId = function () {
    for (var i=1; i < 100; i++) {
        if (!ids.includes(i.toString()))
            return i.toString();
    }
};

const client = init_client(
    broker,
    function onConnect() {
        console.log('MQTT Connected');
        // client.subscribe(PROJECT, {});
        client.subscribe(TOPIC);
    },
    function onMessage(topic, message) {
        console.log('Recv: ', topic, message);
        if (topic === PROJECT) {
            client.unsubscribe(topic);
            onProject(message);
        } else if (topic.startsWith(TOPIC.replace('#', ''))) {
            const arr = topic.split('/');
            updateProfile(arr[arr.length - 1], message);
            // } else if (topic.startsWith(LA_STATE.split('+')[0])) {
            //     updateLA(message);
        }
    }
);

function selectProfile(e) {
    if (currentId) {
        client.unsubscribe(TOPIC.replace('#', currentId), {});
    }
    const creating = e.target.id === 'new-profile';
    if (creating) currentId = undefined;
    else currentId = e.target.id;

    markSelected(e.target);
    const form = $('#profile-form')[0];
    // form.type.disabled = !creating;
    // form.operand.disabled = !creating;

    setInputValue(form, 'name', '');
    setInputValue(form, 'type', '');
    setInputValue(form, 'operand', '');

    $('.items .item').each((_, node) => node.remove());

    $('#profiles').show();
    $('#recipes').hide();

    if (currentId) {
        client.subscribe(TOPIC.replace('#', currentId), {});
    }
}

function updateProfile(id, message) {
    !ids.includes(id) ? ids.push(id) : null;

    const $announcement = $('#announcement');
    const items = $announcement.find('label[id=' + id + ']');

    if (!message) {
        if (items.length > 0) items[0].remove();
        if (ids.includes(id)) ids.splice(ids.indexOf(id));

        if (id === currentId) {
            var labels = $announcement.find(' table label').each(function (index, node) {
                node.click();
                return false;
            });

            if (labels.length === 0) {
                currentId = undefined;
                $('#profiles').hide();
            }
        }
    } else {
        if (items.length === 0) {
            // create announcement
            var item = document.createElement('label');
            item.id = id;
            item.textContent = message.name;

            $announcement.find(' table').each(function (index, node) {
                node.insertRow().insertCell().appendChild(item);
            });
            item.addEventListener('click', selectProfile);

            if (currentId === undefined && (!needId || needId == id)) {
                needId = undefined;
                item.click();
            }
        } else {
            // update announcement
            $(items[0]).text(message.name);
        }

        // sort announcement
        const tbody = $announcement.find(' table tbody');
        var rows = tbody.children().detach().get();
        rows.sort(function (a, b) {
            return $(a).find(" label").text().localeCompare($(b).find(" label").text());
        });
        tbody.append(rows);

        // select first announcement
        if (id === currentId) {
            const profileForm = $('#profile-form')[0];
            var key;
            for (key in message) {
                setInputValue(profileForm, key, message[key]);
            }

            $('.items .item').each((_, node) => node.remove());
            for (var datum of message.items) {
                const itemForm = addItem();
                if (datum.hasOwnProperty('minutes') && datum.hasOwnProperty('hours')) {
                    var date = new Date();
                    date.setUTCHours(datum.hours);
                    date.setUTCMinutes(datum.minutes);
                    datum['time'] = date.toLocaleTimeString().substr(0, 5);
                    delete datum['hours'];
                    delete datum['minutes'];
                }
                for (key in datum) {
                    setInputValue(itemForm, key, datum[key]);
                }
            }
        }
    }
}

function saveProfile() {

    function form2data(form, formData, ignoreDisabled) {
        var _valid = form.reportValidity();
        if (_valid) {
            formData = formData ? formData : {};

            function getValue(index, input) {
                if (ignoreDisabled && input.disabled) return;
                if (input.value === '') return;

                if (input.type === 'checkbox') {
                    formData[input.name] = input.checked;
                } else if (input.hasAttribute('required')) {
                    if (input.type === 'time') {
                        var date = new Date();
                        date.setHours(input.value.split(':')[0]);
                        date.setMinutes(input.value.split(':')[1]);
                        [formData['hours'], formData['minutes']] = date.toISOString().substr(11, 5).split(':');
                    } else {
                        formData[input.name] = isNaN(parseInt(input.value)) ? input.value : parseInt(input.value);
                    }
                }
            }

            $(form).find("input").each(getValue);
            $(form).find("select").each(getValue);

        }
        return [formData, _valid];
    }

    var [data, valid] = form2data($('#profile-form')[0], {items: []});

    if (valid) {
        $('.table#' + data.type).find(' .item').each(function (i, item) {
            var [_data, _valid] = form2data(item);
            if (_valid) {
                _data.id = i + 1;
                data.items.push(_data);
            } else {
                console.log('invalid', item);
            }

            valid = valid && _valid;
            if (!valid) return false;
        });
        if (valid) {
            const id = currentId || generateId();
            console.log('Send: ', TOPIC.replace('#', id), data);
            client.send(TOPIC.replace('#', id), JSON.stringify(data), 0, true);
        }
    }
}

function markSelected(target) {
    $(document).find(' .selected').toggleClass('selected');
    $(target).toggleClass('selected');
}

function onTypeChange(form) {
    $('#profiles .items .table').each(function(_, tableDiv) {
        if (tableDiv.id === form.type.value)
            tableDiv.classList.remove('hidden');
        else
            tableDiv.classList.add('hidden');
    });
}

function onGeoTypeChange(form) {
    const hide = (span) => {
        $(span).find(' input').each((_, input) => {
            input.removeAttribute('required');
            input.disabled = true;
        });
    };

    const show = (span) => {
        $(span).find(' input').each((_, input) => {
            input.setAttribute('required', '');
            input.disabled = false;
        });
    };

    $(form).find('.geo-non-utc').each((_, node) => form.type.value === 'UTC' ? hide(node) : show(node));
    $(form).find('.geo-utc').each((_, node) => form.type.value === 'UTC' ? show(node) : hide(node));
}

function onOperandChange(form) {
    const hide = (span) => {
        $(span).hide();
        $(span).find(' input, select').each((_, input) => {
            if (!span.hasAttribute('min')) {
                span.setAttribute('min', input.min);
                input.min = '';
            }
            if (!span.hasAttribute('max')) {
                span.setAttribute('max', input.max);
                input.max = '';
            }
            input.removeAttribute('required');
        });
    };

    const show = (span) => {
        $(span).show();
        $(span).find(' input, select').each((_, input) => {
            if (span.hasAttribute('min')) {
                input.min = span.getAttribute('min');
                span.removeAttribute('min');
            }
            if (span.hasAttribute('max')) {
                input.max = span.getAttribute('max');
                span.removeAttribute('max');
            }
            input.setAttribute('required', '');
        });
    };

    const table = $('.table');

    table.find('.operand-scene').each((_, node) => form.operand.value === 'Level' ? hide(node) : show(node));
    table.find('.operand-level').each((_, node) => form.operand.value === 'Level' ? show(node) : hide(node));
}

function addItem() {
    const profileForm = $('#profile-form')[0];
    var result = null;

    if (profileForm.reportValidity() && profileForm.type.value) {
        $('#profiles .items').find('.table#'+profileForm.type.value).each(function(_, table) {
            const row = document.createElement('form');
            row.innerHTML = $(table).find('.template-row')[0].innerHTML;
            row.classList.add('tr', 'item');
            table.appendChild(row);

            result = row;
            return false;
        });
    }

    return result;
}

function removeItem(btn) {
    $(btn).parents(' form').remove();
}

function setInputValue(owner, name, value) {
    $(owner).find('[name=' + name + ']').each(function (index, input) {
        if (input.type === 'checkbox') {
            input.checked = value;
        } else {
            input.value = value;
        }

        input.dispatchEvent(new Event('change', {target: input}));
    });
}

function removeProfile() {
    if (currentId)
        client.send(TOPIC.replace('#', currentId), '', 0, true);
    else {
        $('#profiles').hide();
    }
}