var stateTopic, replayTopic, currentGuid;
var profiles = {};
var timeIntervalId;
var client;

function createConnection() {
    client = init_client(
        broker,
        function onConnect() {
            console.log('MQTT Connected');
            client.subscribe(PROFILES + '#');

            $('#announcement label.selected, #announcement label').sort((a, b) => {
                return a.classList.contains('selected') ? -1 : b.classList.contains('selected') ? 1 : 0
            }).each((_, label) => {
                setHistory(label, !label.classList.contains('selected'));
                select(label);
                label.scrollIntoView();
                return false;
            })
        },
        function onMessage(topic, message) {
            // console.log('Recv: ', topic, message);
            if (topic === replayTopic) {
                update(message['key'], message);
            } else if (stateTopic && topic.startsWith(stateTopic)) {
                const arr = topic.split('/');
                update(arr[arr.length - 1], message);
            } else if (topic.startsWith(PROFILES)) {
                storeProfile(topic.split(PROFILES)[1], message);
            }
        },
        createConnection
    );
}


$(document).on('change', 'select.funit', onChange);
$(document).on('input', 'input.funit', onChange);

$(document).find('.field-value').each((_, field) => {
    if ($(field).find('.funit').length > 0) {
        const el = document.createElement('span');
        el.className = 'refresh-funit glyphicon glyphicon-refresh';
        el.addEventListener(
            'click',
            () => $(field).find('.funit').each(
                (_, funit) => requestFunit(funit)
            )
        );
        field.appendChild(el);
    }
});

window.addEventListener('popstate', (event) => {
    select(document.getElementById(event.state));
});

createConnection();

function setHistory(label, replace) {
    if (replace) {
        history.replaceState(label.id, '', '?id=' + label.id);
    } else {
        history.pushState(label.id, '', '?id=' + label.id);
    }
}

function onChange(e) {
    const target = e.target;
    if (target.value !== target.getAttribute('hasValue')) {
        target.classList.add('changed');
    } else {
        target.classList.remove('changed');
    }

    if ($('.funit.changed').length === 0) {
        $('#set-btn').addClass('hidden');
    } else {
        $('#set-btn').removeClass('hidden');
    }
}

function select(label) {
    if (stateTopic)
        client.unsubscribe(stateTopic, {});

    if (replayTopic)
        client.unsubscribe(replayTopic, {});

    currentGuid = guid();
    stateTopic = label.getAttribute('state');
    replayTopic = broker['reply'] + currentGuid;

    clearInterval(timeIntervalId);
    $('#properties select.funit').val('');
    $('#properties input.funit').val('');
    $('#properties span.funit').each((_, span) => setSpanValue(span, '-'));
    $('#announcement .selected').removeClass('selected');
    $('#properties .flag-ordinary').removeClass('flag-ordinary');
    $('#properties .flag-extra').removeClass('flag-extra');
    $('#properties .changed').removeClass('changed');
    $('#set-btn').addClass('hidden');
    $('.field-value .funit').removeAttr('hasvalue');

    if (label.hasAttribute('mc')) {
        $('.refresh-funit').hide();
        $('#renew-btn').hide();
    } else {
        $('.refresh-funit').show();
        $('#renew-btn').show();
    }

    label.classList.add('selected');

    client.subscribe(replayTopic, {});
    client.subscribe(stateTopic + '+', {});
}

function update(key, message) {
    let value = message.value,
        invalid = message.invalid;

    if ($('#properties').find('#' + key).each((_, node) => {
        var flag;

        if (key === 'OperTime')
            value = secToPeriod(value);

        if (key === 'Time') {
            if (message.hasOwnProperty('timestamp')) {
                updateTime(new Date(+new Date(value) + (new Date() - new Date(message.timestamp))));
                if (!invalid) {
                    timeIntervalId = setInterval(updateTime, 1000);
                }
            }
        } else if (key === 'Flags') {
            for (flag in value)
                if (value.hasOwnProperty(flag)) {
                    const $flag = $(node).find('#' + flag);
                    value[flag] ? $flag.addClass('flag-ordinary') : $flag.removeClass('flag-ordinary');
                }
        } else if (key === 'ExtraFlags') {
            for (flag in value)
                if (value.hasOwnProperty(flag)) {
                    const $flag = $(node).find('#' + flag);
                    value[flag] ? $flag.addClass('flag-extra') : $flag.removeClass('flag-extra');
                }
        } else if (key === 'CurrentProfile') {
            drawProfile(node, value);
        } else if (node.tagName === 'SPAN') {
            setSpanValue(node, value);
        } else if (node.type === 'checkbox'){
            node.checked = value;
        } else {
            node.value = value;
            node.setAttribute('hasValue', value);
        }

        if (invalid) {
            node.classList.add('invalid');
        } else {
            node.classList.remove('invalid');
        }
    }).length === 0) {
        console.log(key);
    }
}

function setSpanValue(span, value) {
    var ch = Array.from(span.children);
    span.textContent = value;
    while (ch.length > 0)
        span.appendChild(ch.pop());
}

function updateTime(time) {
    try {
        const input = $('#Time');
        time = time || new Date(input.val());
        time.setSeconds(time.getSeconds()+1);
        input.val(time.toLocaleString("sv-SE", {
            year: "numeric",
            month: "2-digit",
            day: "2-digit",
            hour: "2-digit",
            minute: "2-digit",
            second: "2-digit"
        }).replace(" ", "T"));
    } catch (e) {
        console.exception(e);
        clearInterval(timeIntervalId);
    }
}

function secToPeriod(seconds) {
    var days = Math.floor(seconds / (3600*24));
    seconds  -= days*3600*24;
    var hrs   = Math.floor(seconds / 3600);
    seconds  -= hrs*3600;
    var mnts = Math.floor(seconds / 60);
    seconds  -= mnts*60;

    var result = new Date(0,0);
    result.setHours(hrs);
    result.setMinutes(mnts);
    result.setSeconds(seconds);
    return (days > 0 ? days + ' days ' : '') + result.toLocaleString().split(' ')[1];
}

function requestFunit(funit, topic) {
    topic = topic || stateTopic.replace('State', 'Renew');

    funit.value = '';
    client.send(
        topic + funit.id,
        JSON.stringify({
            key: funit.id,
            session: currentGuid,
        }),
    );
}

function setFunit() {
    const topic = stateTopic.replace('State', 'Set');

    $('.changed').each((_, input) => {
        input.classList.remove('changed');
        client.send(
            topic + input.id,
            JSON.stringify({
                value: input.tagName === 'checkbox' ? input.checked : input.value,
                key: input.id,
                session: currentGuid,
            })
        );

        input.value = '';
    });

    $('#set-btn').addClass('hidden');
}

function requestAll() {
    const topic = stateTopic.replace('State', 'Renew');
    if (topic) {
        $('#properties').find('.funit').each((_, input) => requestFunit(input));
    }
}

function storeProfile(id, data) {
    const selector = $("#ActiveProfile");
    var option = selector.find("#profile-" + id);
    if (data) {
        if (option.length === 0) {
            const opt = document.createElement('option');
            opt.id = 'profile-' + id;
            opt.value = id;
            selector[0].appendChild(opt);

            option.push(opt);
        }
        if (data.name !== option.text()) {
            option.text(data.name);
        }
    } else {
        option.remove();
    }
}

// drawProfile(document.getElementById('geo'), {
//   "type":"Geo",
//   "operand":"Level",
//   "id":0,
//   "items":[
//     {
//       "type":"Sunset",
//       "value":254,
//       "id":2,
//       "offset":0
//     },
//       {
//       "type":"Sunrise",
//       "value":0,
//       "id":1,
//       "offset":0
//     }
//   ]
// }, 'geo');

// drawProfile(document.getElementById('lum'), {
//   "items":[
//     {
//       "lum":25,
//       "value":254,
//       "id":1
//     },
//     {
//       "lum":50,
//       "value":127,
//       "id":2
//     },
//     {
//       "lum":75,
//       "value":0,
//       "id":3
//     }
//   ],
//   "name":"Профиль 3",
//   "type":"Lum",
//   "operand":"Level"
// }, 'lum');

// drawProfile(document.getElementById('utc'), {
//   "items":[
//     {
//       "hours":"11",
//       "minutes":"30",
//       "value":190,
//       "id":1
//     },
//     {
//       "hours":"13",
//       "minutes":"30",
//       "value":127,
//       "id":2
//     },
//     {
//       "hours":"19",
//       "minutes":"00",
//       "value":10,
//       "id":3
//     }
//   ],
//   "name":"Профиль 2",
//   "type":"UTC",
//   "operand":"Level"
// }, 'utc');

// drawProfile(document.getElementById('CurrentProfile'), {
//     "type":"Temp",
//     "items":[
//         {
//             "period":3600,
//             "value":127
//         }
//     ]
// }, 'utc');
