const TITLES = {
	'Geo': gettext("lom_profiles_name_geo"),
	'Lum': gettext("lom_profiles_name_lum"),
	'UTC': gettext("lom_profiles_name_utc"),
	'Temp': gettext("lom_profiles_name_temp"),
};

class GraphFrame {
	x;
	y;
	width;
	height;
}

// Отрисовка прямой
function DrawLine(ctx, fromX, fromY, toX, toY, lineWidth, strokeStyle) {
	ctx.strokeStyle = strokeStyle;
	ctx.lineWidth = lineWidth;
	ctx.beginPath();
	ctx.moveTo(fromX, fromY);
	ctx.lineTo(toX, toY);
	ctx.stroke();
	ctx.closePath();
}

// Вывод текста
function DrawText(ctx, x, y, textLine, fillStyle, font) {
	ctx.fillStyle = fillStyle;
	ctx.font = font;
	ctx.fillText(textLine, x, y);
}

// Вывод координатной горизонтальной прямой
function DrawCoordinateLine(ctx, graphFrame, headerHeight, y, textLine) {
	DrawLine(ctx, headerHeight, y, graphFrame.x + graphFrame.width, y, 1, "#666666");
	DrawText(ctx, headerHeight / 2 - 20, y + headerHeight / 12, textLine, "#666666", "normal " + (headerHeight/5) + "px Arial");
}

// Отрисовка ломаной по массиву точек
function DrawBrokenLine(ctx, points, lineWidth, strokeStyle) {
	ctx.strokeStyle = strokeStyle;
	ctx.lineWidth = lineWidth;
	ctx.beginPath();
	ctx.moveTo(points[0][0], points[0][1]);
	for(var i = 1; i < points.length; i++) {
		ctx.lineTo(points[i][0], points[i][1]);
		ctx.stroke();
	}
	ctx.closePath();
}

// Отрисовка ломаной графика по массиву значений. Рисуется внутри graphFrame. Начало координат - левый нижний угол graphFrame. Абсцисса от 0 до limitX. Ордината от 0 до limitY.
function DrawGraphBrokenLine(ctx, values, limitX, limitY, graphFrame, lineWidth, strokeStyle, singleLine) {
	var points = [];
	let lastItem = values[values.length - 1];

	const value2point = (value) => {
		return [
			graphFrame.x + value[0] * (graphFrame.width / limitX),
			graphFrame.y + graphFrame.height - (value[1] * (graphFrame.height / limitY))
		]
	};

	if (!singleLine) points.push(value2point([0, lastItem[1]]));

	for(var i = 0; i < values.length; i++) {
		points.push(
			value2point([values[i][0], lastItem[1]]),
			value2point(values[i])
		);

		lastItem = values[i];
	}

	if (!singleLine) points.push(value2point([limitX, lastItem[1]]));

	DrawBrokenLine(ctx, points, lineWidth, strokeStyle);
}

// Рассчет отступов от краев канвы т.е. определение области; вывод заголовка и подвала; вывод координатной сетки; вывод текущего дня и времени (если заданы)
function MakeFrame(graphCanvas, ctx, title, footer, xValues, currentTime) {
	var graphFrame = new GraphFrame();

	// Расчитаем высоту хидера
	var headerHeight = graphCanvas.height / 5;

	// Определим область под график, включая всю координатную сетку
	graphFrame.x = headerHeight / 2;
	graphFrame.y = headerHeight;
	graphFrame.width = graphCanvas.width - headerHeight;
	graphFrame.height = graphCanvas.height - 2 * headerHeight;

	// Вывод заголовка
	DrawText(ctx, headerHeight / 2, headerHeight / 2 + headerHeight / 6, title, "#AAAAAA", "normal " + (headerHeight/3) + "px Arial");

	// Вывод подвала
	DrawText(ctx, headerHeight, graphCanvas.height - headerHeight / 2.2, footer, "#AAAAAA", "normal " + (headerHeight/4) + "px Arial");

	/// Вывод координатной сетки по Y
	for (let i=0; i <= 10; i++) {
		DrawCoordinateLine(ctx, graphFrame, headerHeight, headerHeight + graphFrame.height * i / 10, (100 - i*10 + "%").padStart(4, ' '));
	}

	// Отрежем от области под график подписи горизонтальных линий
	// Т.о. останется область непосредственно самого графика
	graphFrame.x = headerHeight;
	graphFrame.width = graphFrame.width - headerHeight / 2;

	// Вывод координатной сетки по X
	// График - один день цикла. Внизу сетки - часы
	for(let i in xValues) {
		DrawLine(ctx,
			graphFrame.x + i * graphFrame.width / (xValues.length - 1), graphFrame.y + graphFrame.height + headerHeight/20,
			graphFrame.x + i * graphFrame.width / (xValues.length - 1), graphFrame.y + graphFrame.height + headerHeight/20 + headerHeight/10,
			1, "#666666");

		var d; // Это для правильного центрирования чисел разной длины
		if(xValues[i].toString().length === 1) {d = headerHeight/20}
		else if(xValues[i].toString().length === 2) {d = headerHeight/10}
		else {d = headerHeight/5}

		DrawText(ctx,
			graphFrame.x + i * graphFrame.width / (xValues.length - 1) - d,
			graphFrame.y + graphFrame.height + headerHeight/3,
			xValues[i], "#777777", "normal " + (headerHeight/5) + "px Arial");
	}

	DrawLine(ctx,
		graphFrame.x + graphFrame.width, graphFrame.y,
		graphFrame.x + graphFrame.width, graphFrame.y + graphFrame.height,
		1, "#666666");

	DrawLine(ctx,
		graphFrame.x, graphFrame.y,
		graphFrame.x, graphFrame.y + graphFrame.height,
		1, "#666666");

	if(currentTime != null) { // Если задано текущее время в минутах с начала дня рисуем вертикальную черту на графике
		DrawLine(ctx,
			graphFrame.x + graphFrame.width * currentTime / (24 * 60), graphFrame.y,
			graphFrame.x + graphFrame.width * currentTime / (24 * 60), graphFrame.y + graphFrame.height,
			2, "#AAAAAA");
	}

	return graphFrame;
}

// Вывод на канву графика всего цикла
function drawProfile(graphCanvas, profile) {
	let graphFrame, limitX;
	let ctx = graphCanvas.getContext('2d');
	const title = TITLES[profile.type];
	const singleLine = profile.type === 'Temp';


	ctx.clearRect(0, 0, graphCanvas.width, graphCanvas.height);
	if (profile.type === "Lum") {
		limitX = 100;
		graphFrame = MakeFrame(graphCanvas, ctx, title, gettext('lom_profiles_graph_lum'), [0,10,20,30,40,50,60,70,80,90,100]);
	} else {
		limitX = 60*60*24;
		graphFrame = MakeFrame(graphCanvas, ctx, title, gettext('lom_profiles_graph_hours'), [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24]);
	}
	let profileItems = [];

	const applyOffset = function (minutes) {
		var result = minutes - new Date().getTimezoneOffset()*60;
		if (result < 0) result = 24*60*60 + result;
		else if (result > 24*60*60) result = result - 24*60*60;
		return result;
	}

	profile.items.forEach((item) => {
		const level = item.value * 100 / 254;
		if (profile.type === 'UTC' || item.type === 'UTC') {
			profileItems.push([applyOffset(item.hours * 3600 + item.minutes * 60), level]);
		} else if (item.type === 'Sunrise') {
			profileItems.push([applyOffset(SUN.sunrise + item.offset), level]);
		} else if (item.type === 'Sunset') {
			profileItems.push([applyOffset(SUN.sunset + item.offset), level]);
		} else if (profile.type === 'Lum') {
			profileItems.push([item.lum, level]);
		} else if (profile.type === 'Temp') {
			let now = new Date();
			now = now.getHours() * 3600 + now.getMinutes() * 60 + now.getSeconds();
			profileItems.push([now, level], [now + item.period, level]);
		}
	});
	profileItems.sort((a, b) => a[0] === b[0] ? 0 : a[0] > b[0] ? 1 : -1);

	DrawGraphBrokenLine(ctx, profileItems, limitX, 100, graphFrame, 1, "#ff6101", singleLine);
}
