log('mercury/meter.js loaded');

meters = (function (global) {
    var Meter = function (data, client) {
        this.data = data;
        this.client = client;
        this.period = {
            start: {
                date: null,
                data: {}
            },
            end: {
                date: null,
                data: {}
            }
        };

        this.getValue = function(variable, data) {
            if (!data) data = {};
            data.session = this.sessionId;
            data.key = variable.funitType;
            this.sendJocket(
                this.getTopic.replace('{}', variable.funitType),
                JSON.stringify(data)
            );
        };

        this.setValue = function(variable, data) {
            if (!data) data = {};
            data.session = this.sessionId;
            data.key = variable.funitType;
            this.sendJocket(
                this.setTopic.replace('{}', variable.funitType),
                JSON.stringify(data)
            );
        };
        this.sendJocket = function(topic, data) {
            this.client.send(topic, data);
            console.log('command: [' + topic + ']: ' + data);
        };
        this.onSelect = function() {
            this.resetValues(true, true, true);
            this.client.subscribe(this.replyTopic);
            this.getTransformRates();
            this.checkAutoupdatePanel();
            this.getCurrentTime();
            this.fillStoredEnergy();
            this.client.subscribe(this.eventTopic);
        };
        this.unselect = function() {
            if (this.timer) {
                clearInterval(this.timer);
            }
            this.client.unsubscribe(this.replyTopic);
            this.client.unsubscribe(this.eventTopic);
        };
        this.fillTransformV = function(value) {
            const f = ~(value + '').indexOf('.') ? (value + '').split('.')[1].length : 0;
            return (value * this.tV).toFixed(f);
        };
        this.fillTransformA = function(value) {
            const f = ~(value + '').indexOf('.') ? (value + '').split('.')[1].length : 0;
            return (value * this.tA).toFixed(f);
        };
        this.onUpdate = function(topic, jocket) {
            if (jocket.hasOwnProperty('error')) {
                console.log("Error: " + JSON.stringify(jocket.error));
                return;
            }
            topic = topic.split('/');
            var state = null;

            if (topic[0] === 'Spread') {
                if (jocket.hasOwnProperty("error")) {
                    console.log(JSON.stringify(jocket.error));
                } else if (topic[1] === 'State') {
                    state = new funits.State(topic.slice(-1)[0], jocket);
                } else if (topic[1] === 'Reply') {
                    state = new funits.State(jocket.key, jocket);
                }
            }

            if (state) {
                if (state.funitType === funits.RateVoltage.funitType) {
                    this.tV = state.value;
                    this.updateTransformState();
                } else if (state.funitType === funits.RateAmperage.funitType) {
                    this.updateTransformState();
                    this.tA = state.value;
                } else if (state.funitType === funits.CurrentDateTime.funitType) {
                    if (state.time) {
                        const date = new Date(state.time);
                        $('#period_start')[0].min   = (new Date(date.getTime() - 1000*60*60*24*123)).toISOString().substr(0, 10);
                        $('#period_end')[0].min     = (new Date(date.getTime() - 1000*60*60*24*123)).toISOString().substr(0, 10);
                        $('#period_start')[0].max   = (new Date(date.getTime() - 1000*60*60*24)).toISOString().substr(0, 10);
                        $('#period_end')[0].max     = (new Date(date.getTime() - 1000*60*60*24)).toISOString().substr(0, 10);
                        var ts = new Date();
                        this.time = parseInt(state.time) - (ts.getTimezoneOffset() * 60 * 1000);
                        if (this.timer) clearInterval(this.timer);
                        this.timer = setInterval(this.updateTime, 1000, this);
                    }
                } else if (state.funitType === funits.LoadOn.funitType) {
                    if (state.value) {
                        $('.poweronoff .status .poweroff')[0].hidden = state.value;
                        $('.poweronoff .status .poweron')[0].hidden = !state.value;
                        $('.poweronoff .status .poweronoffbutton')[0].textContent =
                            state.value ? gettext('mercury_power_switch_off') : gettext('mercury_power_switch_on');
                        $('.poweronoff .password td')[0].className = state.value ? 'poweron' : 'poweroff';
                        $('.poweronoff .password td')[1].className = state.value ? 'poweron' : 'poweroff';
                        this.load = state.value;
                    }
                } else if (state.funitType === funits.StoredEnergy.funitType) {
                    if (this.period.start.date === state.jocket.params.period) {
                        this.period.start.data[state.jocket.params.tariff] = state.value;
                        this.updatePeriod(state.jocket.params.tariff);
                    } else if  (this.period.end.date === state.jocket.params.period) {
                        this.period.end.data[state.jocket.params.tariff] = state.value;
                        this.updatePeriod(state.jocket.params.tariff);
                    }
                }

                for (var divName of state.divs) {
                    var value = divName[1];

                    if (!isNaN(value)) {
                        if (!this.autoUpdateState) {
                            if (this.states.has(divName[0])) {
                                console.log("skip " + divName[0]);
                                continue;
                            }
                        }

                        this.states.set(divName[0], {
                            cl: state.constructor,
                            vDependent: state.isVoltageRateDependent,
                            aDependent: state.isAmperageRateDependent,
                            value: value
                        });

                        if (state.isAmperageRateDependent) {
                            value = this.fillTransformA(value);
                        }

                        if (state.isVoltageRateDependent) {
                            value = this.fillTransformV(value);
                        }
                    }

                    if (typeof document != 'undefined') {
                        var div = document.getElementById(divName[0]);
                        if (div === null) {
                            // console.log("DIV is null! " + divName[0] + ": " + value);
                        } else {
                            // console.log(divName[0] + ": " + value);
                            div.textContent = value;
                        }
                    }
                }
            }
        };
        this.onUpdateClick = function() {
            console.log("clear states");
            this.resetValues(false, true, true);
            this.states = new Map();
            this.fillStoredEnergy();
            this.getValue(funits.RateAmperage);
            this.getValue(funits.RateVoltage);
            this.getValue(funits.PowerTotal);
            this.getValue(funits.PowerActive);
            this.getValue(funits.PowerReactive);
            this.getValue(funits.Amperage);
            this.getValue(funits.Voltage);
            this.getValue(funits.Frequency);
            this.getValue(funits.Angle);
            this.getValue(funits.PowerRate);
        };
        this.getPeriod = function (e) {
            this.period[e.target.name].data = {};
            this.period[e.target.name].date = e.target.value;
            if (e.target.value) {
                var date = new Date(e.target.value);
                if (e.target.name === 'end') {
                    date = new Date(date.getTime() + 1000 * 60 * 60 * 24);
                }
                this.period[e.target.name].date = date.toISOString().substr(0, 10);
                for (var tariff=0; tariff <= 4; tariff++) {
                    this.updatePeriod(tariff);
                    this.getValue(funits.StoredEnergy, {params: {period: date.toISOString().substr(0, 10), tariff: tariff}});
                }
            }
        };
        this.updateTransformState = function() {
            for (var pair of this.states) {
                var divName = pair[0];
                var obj = pair[1];
                var value = obj.value;

                if (obj.vDependent)
                    value = this.fillTransformV(value);
                if (obj.aDependent)
                    value = this.fillTransformA(value);

                if (obj.vDependent === false && obj.aDependent === false)
                    continue;

                if ( typeof document != 'undefined') {
                    var div = document.getElementById(divName);
                    if (div === null) {
                        // console.log("DIV is null on transform sett! " + divName + ": " + value);
                    }
                    else {
                        // console.log(divName + ": " + value);
                        div.textContent = value;
                    }
                }
            }
        };
        this.checkAutoupdatePanel = function() {
            var state = false;
            if (typeof document != "undefined") {
                state = document.getElementById('box-3').checked;
            }
            this.setAutoupdateState(state);
        };
        this.setAutoupdateState = function(state) {
            this.autoUpdateState = state;
        };
        this.updateTime = function(meter) {
            meter.time += 1000;
            var text = new Date(meter.time).toISOString().replace("T", " ").replace(".000Z", "");
            if ( typeof document != 'undefined') {
                var div = document.getElementById('meter_time');
                div.textContent = text;
            }
        };
        this.fillStoredEnergy = function() {
            var period = 0;
            var month = 0;
            if (typeof document != 'undefined') {

                if ($('#energy_total').hasClass('active'))
                    period = 0;
                else if ($('#energy_year').hasClass('active'))
                    period = $('#yearenergyselector').val();
                else if ($('#energy_month').hasClass('active')) {
                    period = 3;
                    month = $('#monthenergyselector').val()
                } else if ($('#energy_day').hasClass('active'))
                    period = $('#dayenergyselector').val();
            }

            this.getStoredEnergy(period, month);
        };
        this.setLoad = function(password) {
            this.setValue(funits.LoadOn, {value: !this.load, params: {password: hashCode(password)}}, true);
        };
        this.getCurrentTime = function() {
            this.getValue(funits.CurrentDateTime)
        };
        this.getStoredEnergy = function(period, month) {
            if (month === undefined) month = 0;
            for (var tariff=0; tariff <= 4; tariff++) {
                this.getValue(funits.StoredEnergy, {params: {period: period, tariff: tariff, month: month}});
            }
        };
        this.getTransformRates = function() {
            this.getValue(funits.RateVoltage);
        };
        this.updatePeriod = function (tariff) {
            function getValues(stack) {
                if (stack !== undefined) {
                    const keys = Object.keys(stack);
                    var values = [];
                    for (var i=0; i < keys.length; i++) values.push(stack[keys[i]]);
                    return values
                }
                else return ['-', '-', '-', '-'];
            }

            const start = getValues(this.period.start.data[tariff]);
            const end = getValues(this.period.end.data[tariff]);

            document.getElementById('s_energy_period_as_'+tariff).textContent = (end[0] - start[0]).toFixed(3).replace(NaN, '-');
            document.getElementById('s_energy_period_ar_'+tariff).textContent = (end[1] - start[1]).toFixed(3).replace(NaN, '-');
            document.getElementById('s_energy_period_rs_'+tariff).textContent = (end[2] - start[2]).toFixed(3).replace(NaN, '-');
            document.getElementById('s_energy_period_rr_'+tariff).textContent = (end[3] - start[3]).toFixed(3).replace(NaN, '-');
        };
        this.resetValues = function(rStatic, rRateDep, rVals) {
            if (rStatic)
                clear(this.staticDivs);
            if (rRateDep)
                clear(this.rateDependent);
            if (rVals)
                clear(this.valueDivs);

            function clear(divs) {
                if (document) {
                    for (var i=0; i < divs.length; i++) {
                        var div = document.getElementById(divs[i]);
                        if (div) {
                            div.textContent = '-';
                        } else {
                            console.log("on resetValues: DIV is null! " + divName);
                        }
                    }
                }
            }
        }
    };

    Meter.prototype = {
        get id() { return this.data.entity_id; },
        get setTopic() { return this.data.set; },
        get getTopic() { return this.data.get; },
        get eventTopic() { return  this.data.state; },
        get replyTopic() { return  this.data.reply; },
        get sessionId() { return  this.data.session_id; },
        get tV() { return  1; },
        get tA() { return  1; },
        get states() { return  new Map(); },
        get rateDependent() { return [
            'Voltage_phase1', 'Voltage_phase2', 'Voltage_phase3',
            'Amperage_phase1', 'Amperage_phase2', 'Amperage_phase3',

            'PowerTotal_total', 'PowerTotal_phase1', 'PowerTotal_phase2', 'PowerTotal_phase3',
            'PowerActive_total', 'PowerActive_phase1', 'PowerActive_phase2', 'PowerActive_phase3',
            'PowerReactive_total', 'PowerReactive_phase1', 'PowerReactive_phase2', 'PowerReactive_phase3',

            's_energy_all_as_0', 's_energy_all_as_1', 's_energy_all_as_2', 's_energy_all_as_3', 's_energy_all_as_4',
            // 's_energy_all_ar_0', 's_energy_all_ar_1', 's_energy_all_ar_2', 's_energy_all_ar_3', 's_energy_all_ar_4',
            // 's_energy_all_rs_0', 's_energy_all_rs_1', 's_energy_all_rs_2', 's_energy_all_rs_3', 's_energy_all_rs_4',
            // 's_energy_all_rr_0', 's_energy_all_rr_1', 's_energy_all_rr_2', 's_energy_all_rr_3', 's_energy_all_rr_4',

            's_energy_year_as_0', 's_energy_year_as_1', 's_energy_year_as_2', 's_energy_year_as_3', 's_energy_year_as_4',
            's_energy_year_ar_0', 's_energy_year_ar_1', 's_energy_year_ar_2', 's_energy_year_ar_3', 's_energy_year_ar_4',
            's_energy_year_rs_0', 's_energy_year_rs_1', 's_energy_year_rs_2', 's_energy_year_rs_3', 's_energy_year_rs_4',
            's_energy_year_rr_0', 's_energy_year_rr_1', 's_energy_year_rr_2', 's_energy_year_rr_3', 's_energy_year_rr_4',

            's_energy_month_as_0', 's_energy_month_as_1', 's_energy_month_as_2', 's_energy_month_as_3', 's_energy_month_as_4',
            's_energy_month_ar_0', 's_energy_month_ar_1', 's_energy_month_ar_2', 's_energy_month_ar_3', 's_energy_month_ar_4',
            's_energy_month_rs_0', 's_energy_month_rs_1', 's_energy_month_rs_2', 's_energy_month_rs_3', 's_energy_month_rs_4',
            's_energy_month_rr_0', 's_energy_month_rr_1', 's_energy_month_rr_2', 's_energy_month_rr_3', 's_energy_month_rr_4',

            's_energy_day_as_0', 's_energy_day_as_1', 's_energy_day_as_2', 's_energy_day_as_3', 's_energy_day_as_4',
            's_energy_day_ar_0', 's_energy_day_ar_1', 's_energy_day_ar_2', 's_energy_day_ar_3', 's_energy_day_ar_4',
            's_energy_day_rs_0', 's_energy_day_rs_1', 's_energy_day_rs_2', 's_energy_day_rs_3', 's_energy_day_rs_4',
            's_energy_day_rr_0', 's_energy_day_rr_1', 's_energy_day_rr_2', 's_energy_day_rr_3', 's_energy_day_rr_4'
        ];},
        get valueDivs() { return [
            'RateAmperage', 'RateVoltage',
            'PowerRate_total', 'PowerRate_phase1', 'PowerRate_phase2', 'PowerRate_phase3',
            'Angle_phase1', 'Angle_phase2', 'Angle_phase3',
            'Frequency'
        ];},
        get staticDivs() { return ['Info_address', 'SerialDate_serial']; },
    };

    return {
        Meter: Meter,
    };
})(window);