"""spread_dashboard URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.urls import path, include
from django.views.i18n import JavaScriptCatalog
from django.contrib import admin

import spread_dashboard.dali.views as dali
import spread_dashboard.lom.views as lom
import spread_dashboard.mercury.views as mercury
import spread_dashboard.monitor.views as monitor
import spread_dashboard.report.views as report
import spread_dashboard.project.views as bam
import spread_dashboard.scheduler.views as scheduler
import spread_dashboard.incubator.views as incubator
import spread_dashboard.local as local
import spread_dashboard.upgrader as upgrader
from spread_dashboard.utils import app
from spread_core.tools.settings_const import HTTP_ROOT

HTTP_ROOT = app().static_settings[HTTP_ROOT]


@login_required
def index(request):
    return render(request, 'index.html', dict(inited=app().has_project))


urlpatterns = [
    path(f'{HTTP_ROOT}/scheduler/month/<int:m>/<int:y>/', scheduler.month),
    path(f'{HTTP_ROOT}/scheduler/browser/', scheduler.browser),
    path(f'{HTTP_ROOT}/scheduler/<str:api_name>/', scheduler.on_api),
    path(f'{HTTP_ROOT}/scheduler/', scheduler.page),

    path(f'{HTTP_ROOT}/mercury/<int:meter_id>/', mercury.page),
    path(f'{HTTP_ROOT}/mercury/check_password/', mercury.check_password),
    path(f'{HTTP_ROOT}/mercury/<str:api_name>/', mercury.on_api),
    path(f'{HTTP_ROOT}/mercury/', mercury.page),

    path(f'{HTTP_ROOT}/dali/', dali.DaliDimmer.as_view()),
    path(f'{HTTP_ROOT}/dali/<int:id>/', dali.DaliDimmer.as_view()),

    path(f'{HTTP_ROOT}/lom/', lom.LomDimmer.as_view()),

    path(f'{HTTP_ROOT}/monitor/dali/', monitor.Dali.as_view()),
    path(f'{HTTP_ROOT}/monitor/', monitor.index),

    path(f'{HTTP_ROOT}/project/ctp/', bam.ctp),
    path(f'{HTTP_ROOT}/project/reread_ctp/', bam.reread_ctp),
    path(f'{HTTP_ROOT}/project/<str:file_name>/', bam.config),
    path(f'{HTTP_ROOT}/project/', bam.index),

    path(f'{HTTP_ROOT}/incubator/cycles/edit/<int:cid>/', incubator.cycle),
    path(f'{HTTP_ROOT}/incubator/cycles/', incubator.cycles),
    path(f'{HTTP_ROOT}/incubator/', incubator.locations),


    path(f'{HTTP_ROOT}/local/equipment/lighting/la/buttons/', local.equipment.lighting.la.buttons.index),
    path(f'{HTTP_ROOT}/local/hardware/lom/<int:profile_id>/', local.hardware.lom.profiles.index),
    path(f'{HTTP_ROOT}/local/hardware/lom/', local.hardware.lom.profiles.index),

    path(f'{HTTP_ROOT}/report/', report.Report.as_view()),

    path(f'{HTTP_ROOT}/upgrade/', upgrader.upgrade),

    path(f'{HTTP_ROOT}/admin/', admin.site.urls),
    path(f'{HTTP_ROOT}/auth/', include('django.contrib.auth.urls')),
    path(f'{HTTP_ROOT}/auth2/', include('spread_dashboard.auth2.urls')),

    path(f'{HTTP_ROOT}/jsi18n/', JavaScriptCatalog.as_view(), name='javascript-catalog'),

    path(f'{HTTP_ROOT}/', index),
]
