import argparse
import datetime
import json
import re
import sys
import urllib.parse
import urllib.request

import requests

HOST = 'server.air-bit.eu'
URL = 'https://%s' % HOST

cookie = {
    'per_page_modal': 0,
    'per_page': 0,
    'form_advanced': 1,
    'session': '.eJw9kMFqwzAQRH-l6JyDrUaXQC7BxsSwa2zkiN1LaF27iiy1ELckUci_V-TQw5ze8GDmLo7TeVys2ExvfhlX4nj6EJu7eHkXG8Ghc2zKnCuQUMEN4z7nsM8g8Aymm1GWa0wdjG1sivLKoc8o8aZo1xRKhZIt6p0F2atG154DOwhoMTHWlIMbJEm0EA4zOLg0hhS5IWIcXrFILj1cGt15igePDhQWXSBzsCkzSZBc1RZdr8i0W_FYiWE5T8ef73n8-p8AhfdJl0HR5xjTHF070p9XNOzQoAe9O1Hob2DajFx_4Qpd026fut9lPD_vEFKJxx-QqGEf.FKsDBQ.DWdst5DXwWMSk29gM6GwEkXawU8',
}

headers = {
    'Accept': '*/*',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3',
    'Connection': 'keep-alive',
    'Cookie': '; '.join([f'{key}={value}' for key, value in cookie.items()]),
    'Host': HOST,
    'Sec-Fetch-Dest': 'empty',
    'Sec-Fetch-Mode': 'no-cors',
    'Sec-Fetch-Site': 'same-origin',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:95.0) Gecko/20100101 Firefox/95.0',
    'X-Requested-With': 'XMLHttpRequest',
}


def keyboard_interrupted(func):
    def inner(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except KeyboardInterrupt:
            pass
    return inner


def add_queue(args):
    try:
        bytes.fromhex(args.payload)
    except:
        raise BaseException('"payload" ожидается, как hex-строка')

    data = {
        'data': args.payload,
        'mac': '',
        'fport': args.fport,
        'confirmed': args.conf,
        'immediate': args.imme,
        'no_join_del': args.join
    }
    url = urllib.parse.unquote(f'{URL}/enddevices/queue/%s/?' % args.id)

    count = 0
    while count < args.length:
        try:
            requests.post(url, data=data, headers=headers)
            count += 1
            sys.stdout.write(f'\rCount: {count} of {args.length} ')
        except KeyboardInterrupt:
            break


def clear_queue(args):
    end_id = args.end_id

    if end_id is None:
        end_id = args.start_id

    start_time = datetime.datetime.now()
    result = dict()
    print(start_time)

    try:
        for dev_id in range(args.start_id, end_id + 1):
            url = urllib.parse.unquote(f'{URL}/enddevices/queue/{dev_id}/')
            content = requests.get(url, headers=headers).content.decode()
            items = re.findall(r'/enddevices/queue/delete/(\d+)/\?next_url', content)
            sys.stdout.write(f'\r{dev_id}: 0 из {len(items)}')
            sys.stdout.flush()
            if items:
                if args.idle is False:
                    result[dev_id] = len(items)
                    i = 0
                    for item in items:
                        i += 1
                        sys.stdout.write(f'\r{dev_id}: {i} из {len(items)}')
                        sys.stdout.flush()
                        requests.get(urllib.parse.unquote(f'{URL}/enddevices/queue/delete/{item}/'), headers=headers)
                print()
    finally:
        if args.idle is False:
            with open('/home/pi/queue.air-bit.%s' % int(start_time.timestamp()), 'w+') as fs:
                json.dump(result, fs, indent=2)


@keyboard_interrupted
def run():
    parser = argparse.ArgumentParser(description='Some AirBit actions of web-API')
    subparsers = parser.add_subparsers()

    _parser = subparsers.add_parser(clear_queue.__name__)
    _parser.add_argument('start_id', help="Начало диапозона ID устройств", default=1235, type=int)
    _parser.add_argument('-end_id', help="Конец диапозона ID устройств", default=None, type=int)
    _parser.add_argument('--idle', action='store_true', help="Не удалять очередь (только оображение)")
    _parser.set_defaults(func=clear_queue)

    _parser = subparsers.add_parser(add_queue.__name__)
    _parser.add_argument('id', help="ID устройства", type=int)
    _parser.add_argument('-p', '--payload', help="Пакет в формате HEX", default='810C', metavar='<HEX-строка>')
    _parser.add_argument('-l', '--length', help="Желаемая длина очереди", default=100, type=int, metavar='<int>')
    _parser.add_argument('--fport', help='Порт, на который будет отправлен пакет', default=1, type=int, metavar='<int>')
    _parser.add_argument('--conf', action='store_true', help='Отправка с подверждением')
    _parser.add_argument('--imme', action='store_true', help='Немедленная отправка')
    _parser.add_argument('--join', action='store_true', help='Не удалять после join')
    _parser.set_defaults(func=add_queue)

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    run()
